import * as React from "react";
import * as RDS from "../../common-types/RemoteDataState";
import { ITracker } from "../../pacman/record";
import * as TallymanPin_V1 from "../tallyman.v1-pin/HOC";
import * as TallymanMo_V1 from "../tallyman.v1-mo/HOC";
import * as TallymanMoRedir_V1 from "../tallyman.v1-moredir/HOC";
import * as BupperOneClick from "../bupper-one-click/HOC";
import * as TallymanClick2sms_V1 from "../tallyman.v1-click2sms/HOC";
import { IActions } from "./strategy-types";
export declare type ConfigDetails = {
    host: string;
    slug: string;
    country: string;
    device?: "smart" | "feature";
    queryString?: Array<[string, string]>;
    automaticallySubmitAllOperators?: boolean;
    operators?: string[];
    service: string;
};
export declare type IFlow = {
    flow: "pin" | "mo" | "oneClick" | "click2sms" | "moRedir";
    operator?: string;
    flowConfig: ConfigDetails;
    default?: IFlow;
};
export declare type StrategyConfig = {
    operators: IFlow;
    default: IFlow;
};
export declare type IStrategy = {
    strategy: IStrategyNames;
    country: string;
    strategyConfigs: StrategyConfig;
};
declare type IStrategyNames = "pin-mo-ask-operator" | "pin-mo-ask-mobile-number" | "pin-one-click-ask-operator" | "pin-one-click-detect-operator-by-ip" | "one-click-mo-pin-detect-operator-by-ip-or-msisdn" | "pin-click2sms-ask-operator" | "mo-redir-one-click-detect-operator-by-ip" | "pin-mo-redir-ask-mobile-number" | "pin-click2sms-ask-mobile-number" | "pin-click2sms-mo-ask-operator" | "click2sms-mo-ask-operator" | "click2sms-mo-ask-mobile-number";
export declare type IConfig = {
    offer: number;
    slug?: string;
    handle?: string;
    country?: string;
    host?: string;
    device?: "smart" | "feature";
    automaticallySubmitAllOperators?: boolean;
    operators?: string[];
};
export interface IHash {
    [key: string]: string;
}
export declare type MSISDNEntryFailure = {
    errorType: MSISDNEntryErrorTypes;
    error: any;
};
export declare type MSISDNEntrySuccess = {};
export declare type OPERATORSelectFailure = {};
export declare type OPERATORSelectSuccess = {
    operators: Array<string>;
    msisdn?: string;
};
export declare type PINEntrySuccess = {
    finalUrl: string;
};
export declare type State = {
    type: "OPERATORSelect";
    result: RDS.RemoteDataState<OPERATORSelectFailure, OPERATORSelectSuccess>;
} | {
    type: "MSISDNEntry";
    result: RDS.RemoteDataState<MSISDNEntryFailure, MSISDNEntrySuccess>;
} | {
    type: "PIN";
    result: (C: React.ComponentType<TallymanPin_V1.HOCProps>) => JSX.Element;
} | {
    type: 'MO';
    result: (C: React.ComponentType<TallymanMo_V1.HOCProps>) => JSX.Element;
} | {
    type: 'CLICK2SMS';
    result: (C: React.ComponentType<TallymanClick2sms_V1.HOCProps>) => JSX.Element;
} | {
    type: 'ONECLICK';
    result: (C: React.ComponentType<BupperOneClick.HOCProps>) => JSX.Element;
} | {
    type: 'MOREDIR';
    result: (C: React.ComponentType<TallymanMoRedir_V1.HOCProps>) => JSX.Element;
} | {
    type: 'LOADING';
    result: RDS.RemoteDataState<{}, {}>;
};
export declare type MSISDNEntryErrorTypes = "AlreadySubscribed" | "UnknownError" | "InvalidMSISDN";
export declare const initialState: State;
export declare type HOCProps = {
    currentState: State;
    actions: IActions;
    operators?: Array<string>;
};
export interface IExtraStrategyParams {
    "oneClick": BupperOneClick.ILinkExtraParams;
}
export declare function strategy<R>({ operatorSelect, msisdnEntry, pinEntry, mo, oneClick, click2sms, moRedir, loading }: {
    msisdnEntry: (rds: RDS.RemoteDataState<MSISDNEntryFailure, MSISDNEntrySuccess>) => R;
    operatorSelect: (rds: RDS.RemoteDataState<OPERATORSelectFailure, OPERATORSelectSuccess>) => R;
    pinEntry: (p: ((C: React.ComponentType<TallymanPin_V1.HOCProps>) => JSX.Element)) => R;
    mo: (p: ((C: React.ComponentType<TallymanMo_V1.HOCProps>) => JSX.Element)) => R;
    oneClick: (p: ((C: React.ComponentType<BupperOneClick.HOCProps>) => JSX.Element)) => R;
    click2sms: (p: ((C: React.ComponentType<TallymanClick2sms_V1.HOCProps>) => JSX.Element)) => R;
    moRedir: (p: ((C: React.ComponentType<TallymanMoRedir_V1.HOCProps>) => JSX.Element)) => R;
    loading: (rds: RDS.RemoteDataState<{}, {}>) => R;
}): (state: State) => R;
export declare const isMSISDNEntry: (s: State) => boolean;
export declare const isPINFlow: (s: State) => boolean;
export declare const isMOFlow: (s: State) => boolean;
export declare const isOneClickFlow: (s: State) => boolean;
export declare const isClic2SmsFlow: (s: State) => boolean;
export declare const isMoRedirFlow: (s: State) => boolean;
export declare const isTopLoading: (s: State) => boolean;
export declare type IResult = {
    operator: string;
    flow: string;
    rockman_id: string;
    message: string;
    success: boolean;
};
export declare function determineFlowByMsidn(internationalMSISDN: string): Promise<IFlow>;
export declare function determineFlowByOperator(operator: string): IFlow;
export declare function getOperators(): Promise<Array<string>>;
export declare const mockedPinFlow: State;
export declare const mockedMOFlow: State;
export declare const mockedOneClickFlow: State;
export declare const mockedClick2smsFlow: State;
export declare const mockedMORedirFlow: State;
declare const _default: <P extends HOCProps>(tracker: ITracker, Comp: React.ComponentType<P>, extraStrategyParams?: IExtraStrategyParams) => (initialState: State, isMocked?: boolean) => {
    new (props: any): {
        state: {
            currentState: State;
            actions: IActions;
            operators?: Array<string>;
            strategyConfig?: IFlow;
        };
        moRedirOneClickDetectOperatorByIp(): Promise<void>;
        pinOneClickDetectOperatorByIp(): Promise<void>;
        pinOneClickMoDetectOperatorByIpOrMsisdn(): Promise<void>;
        askOperator(): Promise<void>;
        componentWillMount(): Promise<void>;
        render(): JSX.Element;
        context: any;
        setState<K extends never>(state: {} | ((prevState: Readonly<{}>, props: Readonly<P>) => {} | Pick<{}, K>) | Pick<{}, K>, callback?: () => void): void;
        forceUpdate(callback?: () => void): void;
        readonly props: Readonly<P> & Readonly<{
            children?: React.ReactNode;
        }>;
        refs: {
            [key: string]: React.ReactInstance;
        };
        componentDidMount?(): void;
        shouldComponentUpdate?(nextProps: Readonly<P>, nextState: Readonly<{}>, nextContext: any): boolean;
        componentWillUnmount?(): void;
        componentDidCatch?(error: Error, errorInfo: React.ErrorInfo): void;
        getSnapshotBeforeUpdate?(prevProps: Readonly<P>, prevState: Readonly<{}>): any;
        componentDidUpdate?(prevProps: Readonly<P>, prevState: Readonly<{}>, snapshot?: any): void;
        UNSAFE_componentWillMount?(): void;
        componentWillReceiveProps?(nextProps: Readonly<P>, nextContext: any): void;
        UNSAFE_componentWillReceiveProps?(nextProps: Readonly<P>, nextContext: any): void;
        componentWillUpdate?(nextProps: Readonly<P>, nextState: Readonly<{}>, nextContext: any): void;
        UNSAFE_componentWillUpdate?(nextProps: Readonly<P>, nextState: Readonly<{}>, nextContext: any): void;
    };
    contextType?: React.Context<any>;
};
export default _default;
