import * as React from "react";
import { IConfig, IHash } from "./main";
import * as RDS from "../../common-types/RemoteDataState";
import { ITracker } from "../../pacman/record";
import { IFlow } from "../super-strategy/HOC";
export declare type MSISDNEntryFailure = {
    errorType: MSISDNEntryErrorTypes;
    error: any;
    productUrl?: string;
};
export declare type MSISDNEntrySuccess = {};
export declare type PINEntryFailure = {
    errorType: PINEntryErrorTypes;
    error?: Error;
};
export declare type PINEntrySuccess = {
    finalUrl: string;
    msisdn?: string;
};
export declare type State = {
    type: "MSISDNEntry";
    result: RDS.RemoteDataState<MSISDNEntryFailure, MSISDNEntrySuccess>;
} | {
    type: "PINEntry";
    actualPIN?: string;
    result: RDS.RemoteDataState<PINEntryFailure, PINEntrySuccess>;
} | {
    type: 'Completed';
    result: PINEntrySuccess;
};
export declare type MSISDNEntryErrorTypes = "AlreadySubscribed" | "UnknownError" | "InvalidMSISDN";
export declare type PINEntryErrorTypes = "UnknownError" | "TooEarly" | "InvalidPIN";
export declare const initialState: State;
export declare const mockedPINState: State;
export declare const mockedCompletedState: State;
export interface IActions {
    submitMSISDN: (window: Window, config: IConfig, msisdn: string, strategy?: IFlow, extraParams?: IHash) => any;
    submitPIN: (pin: string) => any;
    backToStart: () => void;
}
export declare type HOCProps = {
    currentState?: State;
    actions?: IActions;
    strategyConfig?: IFlow;
};
export declare type PINEntryState = RDS.RemoteDataState<PINEntryFailure, PINEntrySuccess>;
export declare type MSISDNEntryState = RDS.RemoteDataState<MSISDNEntryFailure, MSISDNEntrySuccess>;
export declare function match<R>({ msisdnEntry, pinEntry, completed }: {
    msisdnEntry: (rds: RDS.RemoteDataState<MSISDNEntryFailure, MSISDNEntrySuccess>) => R;
    pinEntry: (rds: RDS.RemoteDataState<PINEntryFailure, PINEntrySuccess>) => R;
    completed: (result: PINEntrySuccess) => R;
}): (state: State) => R;
export declare function matchMSISDNEntry<R>(m: RDS.IMatcher<MSISDNEntryFailure, MSISDNEntrySuccess, R>): (model: RDS.RemoteDataState<MSISDNEntryFailure, MSISDNEntrySuccess>) => R;
export declare function matchPINEntry<R>(m: RDS.IMatcher<PINEntryFailure, PINEntrySuccess, R>): (model: RDS.RemoteDataState<PINEntryFailure, PINEntrySuccess>) => R;
export declare const isMSISDNEntry: (s: State) => boolean;
export declare const isPINEntry: (s: State) => boolean;
export declare function whenMSISDNEntry<R>(f: (x: MSISDNEntryState) => R): (s: State) => R;
export declare function whenPINEntry<R>(f: (x: PINEntryState) => R): (s: State) => R;
declare const _default: <P extends HOCProps>(tracker: ITracker, Comp: React.ComponentType<P>, strategy?: IFlow) => (initialState: State) => {
    new (props: any): {
        state: {
            currentState: State;
            actions: IActions;
        };
        render(): JSX.Element;
        context: any;
        setState<K extends never>(state: {} | ((prevState: Readonly<{}>, props: Readonly<P>) => {} | Pick<{}, K>) | Pick<{}, K>, callback?: () => void): void;
        forceUpdate(callback?: () => void): void;
        readonly props: Readonly<P> & Readonly<{
            children?: React.ReactNode;
        }>;
        refs: {
            [key: string]: React.ReactInstance;
        };
        componentDidMount?(): void;
        shouldComponentUpdate?(nextProps: Readonly<P>, nextState: Readonly<{}>, nextContext: any): boolean;
        componentWillUnmount?(): void;
        componentDidCatch?(error: Error, errorInfo: React.ErrorInfo): void;
        getSnapshotBeforeUpdate?(prevProps: Readonly<P>, prevState: Readonly<{}>): any;
        componentDidUpdate?(prevProps: Readonly<P>, prevState: Readonly<{}>, snapshot?: any): void;
        componentWillMount?(): void;
        UNSAFE_componentWillMount?(): void;
        componentWillReceiveProps?(nextProps: Readonly<P>, nextContext: any): void;
        UNSAFE_componentWillReceiveProps?(nextProps: Readonly<P>, nextContext: any): void;
        componentWillUpdate?(nextProps: Readonly<P>, nextState: Readonly<{}>, nextContext: any): void;
        UNSAFE_componentWillUpdate?(nextProps: Readonly<P>, nextState: Readonly<{}>, nextContext: any): void;
    };
    contextType?: React.Context<any>;
};
export default _default;
