// HSV (1978) = H: Hue / S: Saturation / V: Value
var Color = {};
Color.HSV_RGB = function (o) {
    var H = o.H / 360,
        S = o.S / 100,
        V = o.V / 100,
        R, G, B;
    var A, B, C, D;
    if (S == 0) {
        R = G = B = Math.round(V * 255);
    } else {
        if (H >= 1) H = 0;
        H = 6 * H;
        D = H - Math.floor(H);
        A = Math.round(255 * V * (1 - S));
        B = Math.round(255 * V * (1 - (S * D)));
        C = Math.round(255 * V * (1 - (S * (1 - D))));
        V = Math.round(255 * V);
        switch (Math.floor(H)) {
            case 0:
                R = V;
                G = C;
                B = A;
                break;
            case 1:
                R = B;
                G = V;
                B = A;
                break;
            case 2:
                R = A;
                G = V;
                B = C;
                break;
            case 3:
                R = A;
                G = B;
                B = V;
                break;
            case 4:
                R = C;
                G = A;
                B = V;
                break;
            case 5:
                R = V;
                G = A;
                B = B;
                break;
        }
    }
    return {
        R: R,
        G: G,
        B: B
    };
};

var canvas1 = document.createElement("canvas");
var ctx = canvas1.getContext("2d");
var width = 160;
var halfWidth = width / 2;
var rot = (1 / 360) * Math.PI * 2;
var offset = 0;
var oleft = -5;
var otop = -5;
for (var n = 0; n <= 359; n++) {
    var gradient = ctx.createLinearGradient(oleft + halfWidth, otop, oleft + halfWidth, otop + halfWidth);
    var color = Color.HSV_RGB({H: (n + 300) % 360, S: 100, V: 100});
    gradient.addColorStop(0, "rgba(0,0,0,0)");
    gradient.addColorStop(0.7, "rgba(" + color.R + "," + color.G + "," + color.B + ",1)");
    gradient.addColorStop(1, "rgba(255,255,255,1)");
    ctx.beginPath();
    ctx.moveTo(oleft + halfWidth, otop);
    ctx.lineTo(oleft + halfWidth, otop + halfWidth);
    ctx.lineTo(oleft + halfWidth + 6, otop);
    ctx.fillStyle = gradient;
    ctx.fill();
    ctx.translate(oleft + halfWidth, otop + halfWidth);
    ctx.rotate(rot);
    ctx.translate(-(oleft + halfWidth), -(otop + halfWidth));
}


var canvas2 = document.createElement("canvas");
var ctx = canvas2.getContext("2d");
ctx.save();
ctx.globalCompositeOperation = "lighter";
ctx.beginPath();
ctx.fillStyle = "rgba(255,0,0,1)";
ctx.arc(50, 100, 50, Math.PI * 2, 0, false);
ctx.fill()
ctx.beginPath();
ctx.fillStyle = "rgba(0,0,255,1)";
ctx.arc(110, 100, 50, Math.PI * 2, 0, false);
ctx.fill()
ctx.beginPath();
ctx.fillStyle = "rgba(0,255,0,1)";
ctx.arc(80, 50, 50, Math.PI * 2, 0, false);
ctx.fill();

export default function canvasHash() {

    var canvas = document.createElement("canvas");

    if (!(canvas.getContext && canvas.getContext('2d'))) {
        return null
    }
    canvas.width = 200;
    canvas.height = 150;
    canvas.style.display = 'inline';
    var ctx = canvas.getContext("2d");

    ctx.rotate(10 * Math.PI / 180);
    ctx.drawImage(canvas1, 20, -20, 250, 150);
    ctx.globalCompositeOperation = 'hue';
    ctx.drawImage(canvas2, 20, -20, 250, 150);
    ctx.globalCompositeOperation = "source-over";
    ctx.fillStyle = "rgba(0,0,0,0.8)";


    ctx.textBaseline = 'alphabetic'
    ctx.fillStyle = '#f60'
    ctx.fillRect(125, 54, 62, 20)
    ctx.fillStyle = '#069'
    ctx.font = '15pt Arial'
    ctx.strokeText('Cwm fjordban quiz, \ud83d\ude03', 10, 78)
    ctx.fillStyle = 'rgba(102, 204, 0, 1)'
    ctx.font = '18pt Arial'
    ctx.fillText('Cwm fjor, \ud83d\ude03', 4, 45)

    return canvas.toDataURL();
}