import { IFlow } from "../super-strategy/HOC";
export type IConfig = {
    host?: string, country?: string, slug?:string, offer: number
}
  

export type IKeywordShortcode = {
  keyword : string,
  shortcode : string
}

export type IResult = {
  isValid: boolean,
  errorText?: string,
  errorType?: string,
  submissionId?: string,
  keyword: IKeywordShortcode
}

const getConfig = ():IConfig=>{
  try {
    const configs:IConfig = require("../../../../../config.json");
    return configs;
  } catch (ex) {
    throw "config.json not found";
  }
};

export default async function submitMSISDN(window: Window, maybeConfig: IConfig, msisdn: string, strategy?:IFlow): Promise<IKeywordShortcode> {
  const config: IConfig = strategy ? {
    offer:window.pac_analytics.visitor.offer,
    ...strategy.flowConfig
  } : !maybeConfig ? {offer: window.pac_analytics.visitor.offer} : maybeConfig;
  const { host, country, slug, offer } = !config.host || !config.slug || !config.country ? {offer:window.pac_analytics.visitor.offer,...getConfig()} : config
  const search = window.location.search.substr(1) || ''
  const result : IResult = await fetch(`https://lp-api.sam-media.com/v1/submit_msisdn_mo/${host}/${country}/${slug}/${offer}/?msisdn=${msisdn}&rockman_id=${window.pac_analytics.visitor.rockmanId}&${search}`).then(x => x.json())
  if(!result.isValid) {
    const error = new Error(`${result.errorType}:\n${result.errorText}`)
    error['type'] = result.errorType;
    console.error(error)
    throw error
  } else {
    return result.keyword
  }
}