import {  IFlow } from "../super-strategy/HOC";
export type IConfig = {
    host?: string, country?: string, slug?:string, offer: number
}
  
export type IResult = {
  finalUrl?: string,
  isValid: boolean,
  errorText?: string,
  errorType?: string,
  submissionId?: string
}

export interface IHash {
  [key: string]: string
}

const getConfig = ():IConfig=>{
  try {
    const configs:IConfig = require("../../../../../config.json");
    return configs;
  } catch (ex) {
    throw "config.json not found";
  }
};

export default async function submitMSISDN(window: Window, maybeConfig: IConfig, msisdn: string, strategy?:IFlow, extraParams?: IHash): Promise<(pin: string) => Promise<string>> {
  const config: IConfig = strategy ? {
    offer:window.pac_analytics.visitor.offer,
    ...strategy.flowConfig
  } : !maybeConfig ? {offer: window.pac_analytics.visitor.offer} : maybeConfig
  const { host, country, slug, offer } = !config.host || !config.slug || !config.country ? {offer:window.pac_analytics.visitor.offer, ...getConfig()} : config
  const search = window.location.search.substr(1) || ''
  const extraParamsQs = !extraParams ? '' : '&' + Object.keys(extraParams).map(k => `${k}=${extraParams[k]}`).join('&')

  if(country.toLowerCase() === 'pt'){
    return assrock_pt({ host, country, slug, offer, msisdn, extraParamsQs, search })
  } else {
    return assrock_v1({ host, country, slug, offer, msisdn, extraParamsQs, search })
  }
}

async function assrock_v1({ host, country, slug, offer, msisdn, extraParamsQs, search }: { host: string, country: string, slug: string, offer: number, msisdn: string, extraParamsQs: string, search: string }) {
  const result: IResult = await fetch(`https://lp-api.sam-media.com/v1/submit_msisdn/${host}/${country}/${slug || slug}/${offer}/?msisdn=${msisdn}&rockman_id=${window.pac_analytics.visitor.rockmanId}${extraParamsQs}&${search}`).then(x => x.json())
  if (!result.isValid) {
    const error = new Error(`${result.errorType}:\n${result.errorText}`)
    error['type'] = result.errorType;
    console.error(error)
    throw error
  } else {
    return async function submitPIN(pin: string): Promise<string> {
      const pinResult: IResult = await fetch(`https://lp-api.sam-media.com/v1/submit_pin?pin=${pin}&sid=${result.submissionId}`).then(x => x.json())
      if (!pinResult.isValid) {
        const pinError = new Error(`${pinResult.errorType}:\n${pinResult.errorText}`)
        pinError['type'] = pinResult.errorType;
        throw pinError
      } else {
        return pinResult.finalUrl
      }
    }
  }
}

async function assrock_pt({ host, country, slug, offer, msisdn, extraParamsQs, search }: { host: string, country: string, slug: string, offer: number, msisdn: string, extraParamsQs: string, search: string }) {
  const result: ({ error: string, submission_id: null } | { submission_id: number, error: null}) = await 
    fetch(`https://lp-api.sam-media.com/pt/v1/submit-phone?offer=${offer}&phone=${msisdn}&rockman_id=${window.pac_analytics.visitor.rockmanId}${extraParamsQs}&${search}`).then(x => x.json())
  if (!!result.error) {
    const error = new Error(`${result.error}`)
    error['type'] = result.error;
    console.error(error)
    throw error
  } else {
    return async function submitPIN(pin: string): Promise<string> {
      const pinResult: ({error: string, final_url: null} | {error: null, final_url: string}) = await fetch(`https://lp-api.sam-media.com/pt/v1/submit-pin?pin=${pin}&submission_id=${result.submission_id}`).then(x => x.json())
      if (!!pinResult.error) {
        const pinError = new Error(`${pinResult.error}`)
        pinError['type'] = pinResult.error;
        throw pinError
      } else {
        return pinResult.final_url
      }
    }
  }
}