import { IKeywordShortcode } from "../assrock-mo/main";


type Config = {host?: string, country?: string, slug?: string, offer: number}
export type IConfig = { tag: "bupper"} & Config | {tag: "keywordAndShortCode"} & IKeywordShortcode
export { IKeywordShortcode }


const getConfig = ():Config=>{
  try {
    const configs:Config = require("../../../../../config.json");
    return configs;
  } catch (ex) {
    throw "config.json not found";
  }
};

type IFetchResult = { "success": boolean, "uid": string, "shortcode": string, "keyword": string, message?: string }

export default async function load(window: Window, maybeConfig: IConfig): Promise<IKeywordShortcode> {
  
  if(!!maybeConfig && maybeConfig.tag == "keywordAndShortCode") {
    const {keyword, shortcode} = maybeConfig
    return {keyword, shortcode} as IKeywordShortcode
  } else {
    const config = !maybeConfig ? {offer: window.pac_analytics.visitor.offer} : (maybeConfig as Config)
    const { host, country, slug, offer } = !config.host || !config.slug || !config.country ? {offer:window.pac_analytics.visitor.offer, ...getConfig()} : config
    const rockmanId = window.pac_analytics.visitor.rockmanId
    const search = window.location.search.substr(1) || ''
    const result: IFetchResult = await fetch(`https://${host}/tallyman/v1/?action=oc2sms&country=${country}&slug=${slug}&offerId=${offer}&device=smart&rockman_id=${rockmanId}&${search}`).then(x => x.json())
    if(!result.success) {
      throw result.message;
    } else {
      return {keyword: `${result.keyword} ${result.uid}`, shortcode: result.shortcode}
    }
  }
  
}
// {"success":true,"uid":"*5QADM","shortcode":"1901","keyword":"TIPS"}%  
// curl "http://w1.mozzi.com/spa-api/?country=gh&device=smart&slug=18plus-oc2sms&action=oc2sms"