import * as React from "react"
import { ITracker } from "../../pacman/record";
import { updateUrlParameter } from "../../pacman/queryString";
import { IFlow } from "../super-strategy/HOC";

type IState = "NothingYet"

type IActions = {
  onClick: () => void,
  clickLink:any

}

type ACompType = 
    React.ComponentType<{children: React.ReactNode} & React.HTMLAttributes<HTMLAnchorElement>>

export type HOCProps= {
  currentState: IState;
  actions: IActions;
  ClickLink: ACompType;
  strategyConfig?:IFlow;
  clickUrl?: string;
  trackAdvanceInflow?: () => void;
};


export type IConfig = {
  host?: string, country?: string, slug?: string, queryString?: Array<[string, string]>
}

export interface ILinkExtraParams {
  [key: string]: string
}

const getConfig = ():IConfig=>{
  try {
    const configs:IConfig = require("../../../../../config.json");
    return configs;
  } catch (ex) {
    throw "config.json not found";
  }
};

const getOrOverrideConfig = (maybeConfig: IConfig, strategy?:IFlow) : IConfig => {
  const config: IConfig = strategy ? {
    ...strategy.flowConfig
  } : !maybeConfig ? {} : maybeConfig
  const { host, country, slug, queryString } = getConfig()
  return { host: config.host || host, slug: config.slug || slug, country: config.country || country, queryString: (config.queryString || []).concat(queryString || []), }
} 

const getRedirectUrl = (maybeConfig: IConfig, extraParams?:ILinkExtraParams, strategy?:IFlow): string => {
  if (typeof window == "undefined") {
    return "javascript: console.error('SSR'); void 6"
  } else {
    const { host, country, slug, queryString } = getOrOverrideConfig(maybeConfig, strategy)
    const bupperizeCountry = c => c == 'gb' ? 'uk' : c;
    const offer = window.pac_analytics.visitor.offer
    const s = window.location.search.substr(1)
    const search = (!!s && s.length > 0 ? [s] : []).concat((queryString || []).map(([k, v]) => `${k}=${v}`)).join('&')
    const extraParamsQs = !extraParams ? '' : '&' + Object.keys(extraParams).map(k => `${k}=${extraParams[k]}`).join('&')

    // one click flow must use http not https
    const xcid = window.location.href.split("/")[3].split('?')[0];
    return `http://${host}/${bupperizeCountry(country)}/${slug}?offer=${offer}&atmobirun=1&rockman_id=${window.pac_analytics.visitor.rockmanId}&redirPixels=${window.location.host}&x-xcid=${xcid}${extraParamsQs}&${search}`
  }
}

export function match<R>
  (matcher: {nothingYet : () => R})
  { return (state: IState) =>  state == "NothingYet" ? matcher.nothingYet() : <div>...</div>}


export default (tracker: ITracker, maybeConfig: IConfig, Comp: React.ComponentType<HOCProps>, extraParams?:ILinkExtraParams, strategy?:IFlow) => (initState: IState) => {
  const HOC = class HOC extends React.PureComponent<any,{current: IState, redirectUrl: string | null}> {
    state = {
      current: initState,
      redirectUrl: null
    }
    actions : IActions = {
      onClick: () => {
        const url = this.state.redirectUrl
        tracker.advancedInFlow('bupper-one-click', 'click', {url})
        window.location.href = url
      },
      clickLink:null
    } as IActions; 

    componentDidMount() {
      this.setState({redirectUrl: getRedirectUrl(maybeConfig || {}, extraParams, strategy) })

      // empello token for UK (GB)
      const config = getOrOverrideConfig(maybeConfig, strategy)
      if("gb" == config.country || "uk" == config.country) {
        // add empello script in the UK
        const script = document.createElement("script")
        script.src = "//antifraud.empello.net/sngmnmct/lgdfxglz-live.js"
        document.body.append(script)

        const checkEmpello = () => setTimeout(() => {
          if(typeof window["Empello"] == "undefined") {
            return checkEmpello();
          } else {
            window["Empello"].getToken((token: string) => {
              console.log("empello token", token)
              const url = updateUrlParameter(getRedirectUrl(maybeConfig || {}, extraParams, strategy), "empello_token", token)
              this.setState({redirectUrl: url})
            });
          }
        }, 300);
        checkEmpello()
        //

      }
    }

    render() {
      const self = this;
      const url = this.state.redirectUrl
      const clickLink = ({children, ...props}) => <a {...props} onClick={(...args) => {
        tracker.advancedInFlow('bupper-one-click', 'click', {url})
        !!props.onClick ? props.onClick(...args) : void 6;
      }} href={url}>{children}</a>
      self.actions.clickLink = clickLink;
      return (
        <Comp
          actions={self.actions}
          currentState={self.state.current}
          ClickLink={self.actions.clickLink}
          strategyConfig={strategy ? strategy : undefined}
          clickUrl={url}
          trackAdvanceInflow={() => tracker.advancedInFlow('bupper-one-click', 'click', { url })}
          {...this.props}
        />
      );
    }
  }

  HOC.prototype['_client'] = "bupper-one-click";
  return HOC;
}

export const initialState : IState = "NothingYet"
