import * as React from "react";

export interface IHash {
  [key: string]: string
}

const LocaleContext = React.createContext({
  locale: "en", 
  messages:  {} as IHash, 
  intl: { formatMessage: ({ id, defaultMessage }, values) => ""}
})

export function IntlProvider({ locale, messages, children }: { locale: string, messages: IHash, children: React.ReactChild | JSX.Element[] | JSX.Element}) {
  return (
    <LocaleContext.Provider value={{
      locale, 
      messages,
      intl: { formatMessage: ({ id, defaultMessage }, values) => {
        let defaultMessageString = !!messages && !!messages[id] ?  messages[id] : defaultMessage || "";
        if(!!values){
          const valKeyArr = Object.keys(values);
          if(Array.isArray(valKeyArr)){
            valKeyArr.map((value)=>{
              const str = defaultMessageString;
              const res = str.replace(`{${value}}`, values[value])
              defaultMessageString = res;
            })
          }
        }
        return defaultMessageString 
      }}
    }}>{children}</LocaleContext.Provider>
  );
}

export function Translate({id, values, defaultMessage} : { id: string, values?: object, defaultMessage?: string}) : string {
  const { messages } = React.useContext(LocaleContext)
  return !!messages && !!messages[id] ?  messages[id] : defaultMessage || "" 
}

export function injectIntl(Child: React.ComponentType<any>) {
  return props => <LocaleContext.Consumer>
      {
        ({intl}) => <Child {...props} intl={intl} />
      }
    </LocaleContext.Consumer>
}

export function defineMessages(messageDescriptors) {
  //console.log('messageDescriptors', messageDescriptors)
  return messageDescriptors;
}