import React from "react";

const OptimizeContext = React.createContext("")

type Maybe<T> = {tag: "Nothing"} | { tag: "Just", value: T }
function nothing<T>() : Maybe<T> {
  return {tag: "Nothing"}
}
function just<T>(v: T): Maybe<T> {
  return { tag: "Just", value: v }
}
function maybe<T> (defaultValue: T, m: Maybe<T>) {
  return m.tag === "Nothing"
    ? defaultValue
    : m.value
}

export class Experiment extends React.PureComponent <{
  loader: string;
}, { variant: Maybe<string> }>{
  static defaultProps = {
    loader: "Loading..."
  };

  state = {
    variant: nothing<string>()
  };
    
  //const gtag = (...args)=>{window.dataLayer.push(...args)}
  componentWillMount(){

    if (typeof window != "undefined") {
      window.dataLayer.push({  event: "optimizeEvent" });
      gtag('event', 'optimize.callback', {
        callback: (_vId, experimentId)=>{
          console.log('optimize.callback experimentid', _vId, experimentId)
          const ga = window['ga']
          if (!!ga) {
            const _expId = experimentId;
            const gtm = ga.getAll()[0].get('name')
            ga(`${gtm}.set`, 'exp', _expId.toString() + '.' + _vId.toString());
            ga(`${gtm}.send`, 'event', 'Experiment', 'Trigger', _expId.toString() + '.' + _vId.toString());
            this.setState({
              variant:just(_vId)
            })
          }
        }
      });  
    }
  }
  componentDidMount(){
    setTimeout(() => {
      if(this.state.variant === null || this.state.variant.tag === "Nothing"){
        this.setState({variant: just("0")})
      }
    },750);
  }
  render() {
    return (
      <OptimizeContext.Provider value={maybe("-1", this.state.variant)}>
        {this.props.children}
      </OptimizeContext.Provider>
    );
  }
}

export function Loader(props: { children?: React.ReactNode}) {
  return <Variant id="-1">{props.children}</Variant>
}

export class Variant extends React.Component <{
  id:string
}>{
  render() {
    return (
      <OptimizeContext.Consumer>
        {value => (value === this.props.id ? this.props.children : null)}
      </OptimizeContext.Consumer>
    );
  }
}

export default {
  Experiment,
  Variant
};
