import * as React from "react";
import * as RDS from "../../common-types/RemoteDataState";
import { ITracker } from "../../pacman/record";
import * as TallymanPin_V1 from "../tallyman.v1-pin/HOC";
import * as TallymanMo_V1 from "../tallyman.v1-mo/HOC";
import * as TallymanMoRedir_V1 from "../tallyman.v1-moredir/HOC";
import * as BupperOneClick from "../bupper-one-click/HOC";
import * as TallymanClick2sms_V1 from "../tallyman.v1-click2sms/HOC";
import { IActions } from "./strategy-types";
import tryGetIPRangeName from "../../ips/tryGetIPRangeName"
import mkTracker from "../../pacman/record";



export type ConfigDetails = {
  host: string;
  slug: string;
  country: string;
  device?: "smart" | "feature";
  queryString?:Array<[string, string]>
  // if 'automaticallySubmitAllOperators' is set to true, the client tries to trigger pin for every operator
  automaticallySubmitAllOperators?: boolean;
  operators?: string[];
  service: string;
}

export type IFlow = {
  flow: "pin" | "mo" | "oneClick" | "click2sms" | "moRedir";
  operator?:string;
  flowConfig:ConfigDetails;
  default?:IFlow;
}
export type StrategyConfig = {
  operators:IFlow;
  default:IFlow;
}


export type IStrategy = {
  strategy: IStrategyNames;
  country: string;
  strategyConfigs:StrategyConfig
}


type IStrategyNames = 
 "pin-mo-ask-operator" 
  | "pin-mo-ask-mobile-number" 
  | "pin-one-click-ask-operator" 
  | "pin-one-click-detect-operator-by-ip"
  | "one-click-mo-pin-detect-operator-by-ip-or-msisdn"
  | "pin-click2sms-ask-operator"
  | "mo-redir-one-click-detect-operator-by-ip"
  | "pin-mo-redir-ask-mobile-number"
  | "pin-click2sms-ask-mobile-number"
  | "pin-click2sms-mo-ask-operator"
  | "click2sms-mo-ask-operator"
  | "click2sms-mo-ask-mobile-number";

export type IConfig = {
  offer: number,
  slug?: string;
  handle?:string;
  country?: string;
  host?:string;
  device?: "smart" | "feature";
  automaticallySubmitAllOperators?: boolean;
  operators?: string[];
}


///

export interface IHash {
  [key: string]: string
}

export type MSISDNEntryFailure = {
  errorType: MSISDNEntryErrorTypes;
  error: any;
};
export type MSISDNEntrySuccess = { };
export type OPERATORSelectFailure = { };
export type OPERATORSelectSuccess = {
  operators:Array<string>;
  msisdn?:string;
};

export type PINEntrySuccess = { finalUrl: string };
export type State =
  | {
      type: "OPERATORSelect",
      result: RDS.RemoteDataState<OPERATORSelectFailure, OPERATORSelectSuccess>;
  }
  | {
      type: "MSISDNEntry";
      result: RDS.RemoteDataState<MSISDNEntryFailure, MSISDNEntrySuccess>;
    }
  | {
      type: "PIN";
      result: (C: React.ComponentType<TallymanPin_V1.HOCProps>) => JSX.Element;
    }
  | {
      type: 'MO';
      result: (C: React.ComponentType<TallymanMo_V1.HOCProps>) => JSX.Element;
    } 
  | {
    type: 'CLICK2SMS';
    result: (C: React.ComponentType<TallymanClick2sms_V1.HOCProps>) => JSX.Element;
  }
  | {
      type: 'ONECLICK';
      result: (C: React.ComponentType<BupperOneClick.HOCProps>) => JSX.Element;
    }
  | {
      type: 'MOREDIR';
      result: (C: React.ComponentType<TallymanMoRedir_V1.HOCProps>) => JSX.Element;
    }
  | {
    type: 'LOADING';
    result: RDS.RemoteDataState<{}, {}>;
  };

export type MSISDNEntryErrorTypes =
  | "AlreadySubscribed"
  | "UnknownError"
  | "InvalidMSISDN";


export const initialState : State = { type: "LOADING", result: RDS.NothingYet<{}, {}>()}

// export interface IActions {
//   submitMSISDN: (window: Window, config: IConfig, msisdn: string, operator:string, extraParams?: IHash) => any,
//   backToStart: () => void
// }

export type HOCProps = {
  currentState: State;
  actions: IActions;
  operators?:Array<string>
};


export interface IExtraStrategyParams {
  "oneClick": BupperOneClick.ILinkExtraParams;
}





export function strategy<R>(
  { operatorSelect,msisdnEntry,  pinEntry, mo, oneClick, click2sms, moRedir, loading }: 
  {   msisdnEntry: (rds: RDS.RemoteDataState<MSISDNEntryFailure, MSISDNEntrySuccess>) => R
    , operatorSelect: (rds: RDS.RemoteDataState<OPERATORSelectFailure, OPERATORSelectSuccess>) => R
    , pinEntry: (p: ((C: React.ComponentType<TallymanPin_V1.HOCProps>) => JSX.Element)) => R
    , mo: (p: ((C: React.ComponentType<TallymanMo_V1.HOCProps>) => JSX.Element)) => R
    , oneClick: (p: ((C: React.ComponentType<BupperOneClick.HOCProps>) => JSX.Element)) => R
    , click2sms: (p: ((C: React.ComponentType<TallymanClick2sms_V1.HOCProps>) => JSX.Element)) => R
    , moRedir: (p: ((C: React.ComponentType<TallymanMoRedir_V1.HOCProps>) => JSX.Element)) => R
    , loading: (rds: RDS.RemoteDataState<{},{}>) => R
  }): (state: State) => R {
  return state => {
    switch (state.type) {
      case 'OPERATORSelect':
        return operatorSelect(state.result);
      case 'MSISDNEntry':
        return msisdnEntry(state.result)
      case 'PIN':
        return pinEntry(state.result)
      case 'MO':
        return mo(state.result)
      case 'ONECLICK':
        return oneClick(state.result)
      case 'CLICK2SMS':
        return click2sms(state.result)
      case 'MOREDIR':
        return moRedir(state.result)
      case 'LOADING':
        return loading(state.result)
    
    }
  }
}

export const isMSISDNEntry = (s: State) =>
  s.type == "MSISDNEntry";
export const isPINFlow = (s: State) =>
  s.type == "PIN";
export const isMOFlow = (s: State) =>
  s.type == "MO";
export const isOneClickFlow = (s: State) =>
  s.type == "ONECLICK";
export const isClic2SmsFlow = (s: State) =>
  s.type == "CLICK2SMS";
export const isMoRedirFlow = (s: State) =>
  s.type == "MOREDIR";
  export const isTopLoading = (s: State) =>
  s.type == "LOADING";

export type IResult = {
  operator:string;
  flow: string;
  rockman_id: string;
  message:string;
  success:boolean
}

const getConfig = ():IStrategy=>{
  try {
    const configs:IStrategy = require("../../../../../config.json");
    return configs;
  } catch (ex) {
    throw "config.json not found";
  }
};

const configs = getConfig();
const defaultFlowConfig = configs.strategyConfigs.default;
const operatorsConfig = configs.strategyConfigs.operators;

export async function determineFlowByMsidn (internationalMSISDN: string):Promise<IFlow>{
  if (!internationalMSISDN ) {
    const submissionError = new Error(`Error in submitMSISDN() trigger-pin action`)
    submissionError['type'] = "SEInvalidMSISDN";
    throw submissionError
  }
  const msisdn = internationalMSISDN.match(/\d+/gi).join('') // tallyman API expects international msisdn without any special character

  try{
    const indentifyFlow  = async()=>{
      const config = defaultFlowConfig.flowConfig;
      const { slug, country, device, host } = config;
      const rockmanId = window.pac_analytics.visitor.rockmanId
      const offer = window.pac_analytics.visitor.offer;
      const flowOperatorResult = await fetch(`https://${host}/tallyman/v1/?action=identify&slug=${slug}&country=${country}&msisdn=${msisdn}&device=${device}&offerId=${offer}&rockman_id=${rockmanId}`).then(x => x.json());
      return flowOperatorResult;
    };
    const theResult = await indentifyFlow();

    const flow = operatorsConfig[theResult.operator] ? operatorsConfig[theResult.operator] : null;
    const cannotDetect = theResult.success === false && theResult.message === "Operator could not be found for the given MSISDN";
    const hlrError = theResult.success === false && theResult.message.indexOf("HLR did not find the operator for given MSISDN") !== -1;
    
    
    if(cannotDetect || hlrError){
      const submissionError = new Error(`Error in submitMSISDN() identify flow`)
      submissionError['type'] = "FlowDetectFailed";
      submissionError['msisdn'] = msisdn;
      throw submissionError
    }else if(flow !== null){
      const flowObj = {operator:theResult.operator, default:defaultFlowConfig, ...flow};
      return flowObj
    }else if(flow === null){
      const flowObj = {operator:theResult.operator, ...defaultFlowConfig};
      return flowObj
    }else{
      const submissionError = new Error(`Error in submitMSISDN() trigger-pin action`)
      submissionError['type'] = "SEInvalidMSISDN";
      throw submissionError
    }
  }catch(err){
    throw err;
  }
}






export function determineFlowByOperator (operator: string):IFlow{
    const flowObj = {operator,default:defaultFlowConfig, ...operatorsConfig[operator]};
    return flowObj
}

//if operator from server
//no need for this

export async function getOperators (): Promise<Array<string>> {
  const operators = Object.keys(operatorsConfig);
  return operators
}


//FLOWS

//pin flow
const pinFlow = (tracker:ITracker, config:IFlow, msisdn?:string, extraParams?:IHash)=>{

    const result : (Component: React.ComponentType<TallymanPin_V1.HOCProps>) => JSX.Element = (Component) => { 
    const HOC = TallymanPin_V1.default;
    const Wrapper = ({currentState, actions, strategyConfig} : TallymanPin_V1.HOCProps)=>{
      if(!!msisdn){
        React.useEffect(() => {
          actions.submitMSISDN(window, null, msisdn, null, extraParams)
        }, [])
      }
      return(
        <Component currentState={currentState} actions={actions}  strategyConfig={strategyConfig}/>
      ) 
    }
    const C = HOC(tracker, Wrapper, config)(TallymanPin_V1.initialState)
    return <C />
  }
  return result;
}

//mo flow
const moFlow = (tracker:ITracker, config:IFlow, msisdn?:string)=>{
  const result : (Component: React.ComponentType<TallymanMo_V1.HOCProps>) => JSX.Element = (Component) => { 
    const HOC = TallymanMo_V1.default
    const Wrapper = ({currentState, actions, strategyConfig} : TallymanMo_V1.HOCProps)=>{
      if(!!msisdn){
        React.useEffect(() => {
          actions.submitMSISDN(window, null, msisdn)
        }, [])
      }
      return <Component currentState={currentState} actions={actions}   strategyConfig={strategyConfig}/>
    }
    const C = HOC(tracker, Wrapper, config)(TallymanMo_V1.initialState)
    return <C />
  }
  return result;
}

//One Click flow
const oneClickFlow = (tracker:ITracker, config:IFlow, extraParams?:BupperOneClick.ILinkExtraParams)=>{
  const result : (Component: React.ComponentType<BupperOneClick.HOCProps>) => JSX.Element = (Component) => {
    const HOC = BupperOneClick.default
    const Wrapper = ({currentState, actions, strategyConfig} : BupperOneClick.HOCProps)=>{
      return <Component actions={actions} currentState={currentState} ClickLink={actions.clickLink} strategyConfig={strategyConfig}/>
    }
    const C = HOC(tracker, null, Wrapper, extraParams, config)(BupperOneClick.initialState)
    return <C />
  }
  return result;
}

//Click 2sms flow
const click2smsFlow = (tracker:ITracker, config:IFlow)=>{
  const result : (Component: React.ComponentType<TallymanClick2sms_V1.HOCProps>) => JSX.Element = (Component) => {
    const HOC = TallymanClick2sms_V1.default
    const Wrapper = ({currentState, actions, MOLink, strategyConfig} : TallymanClick2sms_V1.HOCProps)=>{
      return <Component actions={actions} currentState={currentState} MOLink={actions.moLink} strategyConfig={strategyConfig}/>
    }
    const C = HOC(tracker, Wrapper, null, config)(TallymanClick2sms_V1.initialState)
    return <C />
  }
  return result;
}

//moRedir flow
const moRedirFlow = (tracker:ITracker, config:IFlow, msisdn?:string, bupperNumber?:string)=>{
  const result : (Component: React.ComponentType<TallymanMoRedir_V1.HOCProps>) => JSX.Element = (Component) => { 
    const HOC = TallymanMoRedir_V1.default
    const Wrapper = ({currentState, actions, strategyConfig} : TallymanMoRedir_V1.HOCProps)=>{
     console.log("msisdn", msisdn)
     console.log("bupperNumber", bupperNumber)

      if(!!msisdn && !!bupperNumber){
        React.useEffect(() => {
          actions.submitMSISDN(window, null, msisdn, bupperNumber)
        }, []);
      }
      return <Component currentState={currentState} actions={actions} strategyConfig={strategyConfig}/>
    }
    const C = HOC(tracker, Wrapper, config)(TallymanMoRedir_V1.initialState)
    //@ts-ignore
    return <C />
  }
  return result;
}
function mockConfig(flow):IFlow{
  
  const opArr = Object.keys(operatorsConfig);
  const r = opArr.filter((o)=>operatorsConfig[o].flow === flow);
  const operator = r[0];
  const flowObj = {operator,default:defaultFlowConfig, ...operatorsConfig[operator]};
  return flowObj;
}

const mockTracker = mkTracker(
  typeof window != "undefined" ? window : null,
  "xx",
  "demo-strategy"
);
export const mockedPinFlow : State = { type: "PIN", result:pinFlow(mockTracker, mockConfig("pin"))}
export const mockedMOFlow : State = { type: "MO", result:moFlow(mockTracker, mockConfig("mo"))}
export const mockedOneClickFlow : State = { type: "ONECLICK", result:oneClickFlow(mockTracker, mockConfig("oneClick"), {"testextraparams":"1"})}
export const mockedClick2smsFlow : State = { type: "CLICK2SMS", result:click2smsFlow(mockTracker, mockConfig("click2sms"))}
export const mockedMORedirFlow : State = { type: "MOREDIR", result:moRedirFlow(mockTracker, mockConfig("moRedir"))}

//export const mockedCompletedState : State = { type: "Completed", result: {finalUrl: 'https://www.yahoo.com/'} }
export default <P extends HOCProps>(tracker: ITracker, Comp: React.ComponentType<P>, extraStrategyParams?:IExtraStrategyParams) => (
  initialState: State,
  isMocked?:boolean
) => {
  const HOC = class HOC extends React.PureComponent<P> {
    state: {
      currentState: State;
      actions: IActions;
      operators?:Array<string>;
      strategyConfig?:IFlow;
    };
    constructor(props) {
      super(props);
      const self = this;
      this.state = {
        currentState: initialState,
        actions: {
          backToStart: () => {
            this.setState({
              currentState: initialState
            })
          },
          selectOperator : (operator:string, phoneNumber?:string, extraParams?:IHash)=> {
            
            //@ts-ignore
            const msisdn = phoneNumber;
            try {
              const flowObj =  determineFlowByOperator(operator);
              self.setState({
                strategyConfig: flowObj
              })
              switch (flowObj.flow) {
                case "pin":
                  self.setState({
                    currentState: {type: "PIN", result: pinFlow(tracker, flowObj, msisdn, extraParams) } as State,
                  })
                  tracker.advancedInFlow('tallyman/v1', 'pin-flow',{operator})
                  break;
                case "mo":
                  self.setState({
                    currentState: {type: "MO", result: moFlow(tracker, flowObj, msisdn) } as State,
                  })
                  tracker.advancedInFlow('tallyman/v1', 'mo-flow',{operator})
                  break;
                case "oneClick":
                  self.setState({
                    currentState: {type: "ONECLICK", result: oneClickFlow(tracker, flowObj, (!!extraStrategyParams &&  !!extraStrategyParams["oneClick"]) ? extraStrategyParams["oneClick"] : null)} as State,
                  })
                  tracker.advancedInFlow('bupper', 'oneClick-flow',{operator})
                  break;
                case "click2sms":
                    self.setState({
                      currentState: {type: "CLICK2SMS", result: click2smsFlow(tracker, flowObj)} as State,
                    })
                    tracker.advancedInFlow('bupper', 'click2sms-flow',{operator})
                    break;
                default:
                  self.setState({
                    currentState: {type: "PIN", result: pinFlow(tracker, flowObj, msisdn, extraParams) } as State,
                  })
                  tracker.advancedInFlow('tallyman/v1', 'pin-flow',{operator})
                  break;
              }
            }catch (ex) {}
          },
          submitMSISDN: async (...args) => {
            this.setState({
              currentState: {type: "MSISDNEntry", result: RDS.Loading()} as State
            })
            const [,,msisdn, ,extraParams, bupperNumber] = args;
            

            //MSISDN: strategies
            const pinMoAskMobileNumber = (flowObj:IFlow)=>{
              switch (flowObj.flow) {
                case "pin":
                  self.setState({
                    currentState: {type: "PIN", result: pinFlow(tracker, flowObj, msisdn, extraParams) } as State,
                  })
                  tracker.advancedInFlow('tallyman/v1', 'pin-flow',{msisdn})
                  break;
                case "mo":
                  self.setState({
                    currentState: {type: "MO", result: moFlow(tracker, flowObj, msisdn) } as State,
                  })
                  tracker.advancedInFlow('tallyman/v1', 'mo-flow',{msisdn})
                  break;
                default:
                  self.setState({
                    currentState: {type: flowObj.default.flow.toUpperCase(), result: pinFlow(tracker, flowObj.default, msisdn, extraParams) } as State,
                  })
                  tracker.advancedInFlow('tallyman/v1', 'pin-flow',{msisdn})
                  break;
              }
            }

            const pinMoRedirAskMobileNumber = (flowObj:IFlow)=>{
              switch (flowObj.flow) {
                case "pin":
                  self.setState({
                    currentState: {type: "PIN", result: pinFlow(tracker, flowObj, msisdn, extraParams) } as State,
                  })
                  tracker.advancedInFlow('tallyman/v1', 'pin-flow',{msisdn})
                  break;
                case "moRedir":
                  self.setState({
                    currentState: {type: "MOREDIR", result: moRedirFlow(tracker, flowObj, msisdn, bupperNumber) } as State,
                  })
                  tracker.advancedInFlow('tallyman/v1', 'moRedir-flow',{msisdn})
                  break;
                default:
                  self.setState({
                    currentState: {type: flowObj.default.flow.toUpperCase(), result: pinFlow(tracker, flowObj.default, msisdn, extraParams) } as State,
                  })
                  tracker.advancedInFlow('tallyman/v1', 'pin-flow',{msisdn})
                  break;
              }
            }

            const pinClick2SmsAskMobileNumber = (flowObj:IFlow)=>{
              switch (flowObj.flow) {
                case "pin":
                  self.setState({
                    currentState: {type: "PIN", result: pinFlow(tracker, flowObj, msisdn, extraParams) } as State,
                  })
                  tracker.advancedInFlow('tallyman/v1', 'pin-flow',{msisdn})
                  break;
                case "click2sms":
                  self.setState({
                    currentState: {type: "CLICK2SMS", result: click2smsFlow(tracker, flowObj) } as State,
                  })
                  tracker.advancedInFlow('tallyman/v1', 'click2sms-flow',{msisdn})
                  break;
                default:
                  self.setState({
                    currentState: {type: flowObj.default.flow.toUpperCase(), result: pinFlow(tracker, flowObj.default, msisdn, extraParams) } as State,
                  })
                  tracker.advancedInFlow('tallyman/v1', 'pin-flow',{msisdn})
                  break;
              }
            }

            const click2SmsMoAskMobileNumber = (flowObj:IFlow)=>{
              switch (flowObj.flow) {
                case "mo":
                  self.setState({
                    currentState: {type: "MO", result: moFlow(tracker, flowObj, msisdn) } as State,
                  })
                  tracker.advancedInFlow('tallyman/v1', 'mo-flow',{msisdn})
                  break;
                case "click2sms":
                  self.setState({
                    currentState: {type: "CLICK2SMS", result: click2smsFlow(tracker, flowObj) } as State,
                  })
                  tracker.advancedInFlow('tallyman/v1', 'click2sms-flow',{msisdn})
                  break;
                default:
                  self.setState({
                    currentState: {type: flowObj.default.flow.toUpperCase(), result: click2smsFlow(tracker, flowObj.default) } as State,
                  })
                  tracker.advancedInFlow('tallyman/v1', 'click2sms-flow',{msisdn})
                  break;
              }
            }

            

            try {
              const flowObjByMsisdn  =  await determineFlowByMsidn(msisdn);
              const flowObj = operatorsConfig[flowObjByMsisdn.operator] ? flowObjByMsisdn : {operator:flowObjByMsisdn.operator, ...defaultFlowConfig};

              self.setState({
                strategyConfig: flowObj
              })
              //Switch on strategies based on phone number 
              switch (configs.strategy) {
                case "pin-mo-ask-mobile-number":
                  pinMoAskMobileNumber(flowObj);
                  break;
                case "pin-mo-redir-ask-mobile-number":
                  pinMoRedirAskMobileNumber(flowObj);
                  break;
                case "pin-click2sms-ask-mobile-number":
                  pinClick2SmsAskMobileNumber(flowObj);
                  break;
                case "click2sms-mo-ask-mobile-number":
                  click2SmsMoAskMobileNumber(flowObj)
                  break;
                  //More cases come here
                default:
                  pinMoAskMobileNumber(flowObj);
                  break;
              }
            } catch (ex) {
              const errorType: MSISDNEntryErrorTypes =
                "SEAlreadySubscribed" === ex.type
                  ? "AlreadySubscribed"
                  : "SEInvalidMSISDN" == ex.type
                    ? "InvalidMSISDN"
                    : "UnknownError";

              if(ex.type === "FlowDetectFailed"){
                const operators = await getOperators();
                if(operators.length === 1){ 
                  this.setState({
                    currentState: {type: defaultFlowConfig.flow.toUpperCase(), result: pinFlow(tracker,defaultFlowConfig) } as State,
                  })
                }else{
                  this.setState({
                    currentState: {type: "OPERATORSelect", result:RDS.Success({operators, msisdn:ex.msisdn})} as State,
                  })
                }
                tracker.recedeInFlow('assrock/v1', 'msisdn-flow-detection-failure',{msisdn, errorType: errorType || 'UnknownError'})
              }else{
                self.setState({
                  currentState: {
                    type: "MSISDNEntry",
                    result: RDS.Failure<MSISDNEntryFailure, MSISDNEntrySuccess>({
                      errorType: errorType,
                      error: ex
                    })
                  } as State
                });
                tracker.recedeInFlow('assrock/v1', 'msisdn-submission-failure',{msisdn, errorType: errorType || 'UnknownError'})
              }
            }
          }
        }
      };
    }

    //mo-redir-one-click-detect-operator-by-ip
    async moRedirOneClickDetectOperatorByIp(){
      if (typeof window != "undefined") {
        try{
          const operator = await tryGetIPRangeName(window.pac_analytics.visitor);
          const flowObj = determineFlowByOperator(operator ? operator.toUpperCase() : "");
          this.setState({
            strategyConfig: flowObj
          })
          if(operator !== null && flowObj.hasOwnProperty("flow")){  
            switch (flowObj.flow.toUpperCase()) {
              case "ONECLICK":
                this.setState({
                  currentState: {type: "ONECLICK", result: oneClickFlow(tracker,flowObj, (!!extraStrategyParams &&  !!extraStrategyParams["oneClick"]) ? extraStrategyParams["oneClick"] : null) } as State,
                })   
                break;
              case "MOREDIR":
                this.setState({
                  currentState: {type: "MOREDIR", result: moRedirFlow(tracker,flowObj) } as State,
                }) 
              default:
                this.setState({
                  currentState: {type: defaultFlowConfig.flow.toUpperCase(), result: moRedirFlow(tracker,defaultFlowConfig) } as State,
                })
                break;
            }
          }else{
            this.setState({
              currentState: {type: defaultFlowConfig.flow.toUpperCase(), result: moRedirFlow(tracker,defaultFlowConfig) } as State,
            })
          }
        }catch(err){
          this.setState({
            currentState: {type: defaultFlowConfig.flow.toUpperCase(), result: moRedirFlow(tracker,defaultFlowConfig) } as State,
          })
          throw err
        }
      }
    } 
    async pinOneClickDetectOperatorByIp(){
      if (typeof window != "undefined") {
        try{
          const operator = await tryGetIPRangeName(window.pac_analytics.visitor);
          const flowObj = determineFlowByOperator(operator ? operator.toUpperCase() : "");
          this.setState({
            strategyConfig: flowObj
          })
          if(operator !== null && flowObj.hasOwnProperty("flow")){  
            switch (flowObj.flow.toUpperCase()) {
              case "ONECLICK":
                this.setState({
                  currentState: {type: "ONECLICK", result: oneClickFlow(tracker,flowObj, (!!extraStrategyParams &&  !!extraStrategyParams["oneClick"]) ? extraStrategyParams["oneClick"] : null) } as State,
                })   
                break;
              case "PIN":
                this.setState({
                  currentState: {type: "PIN", result: pinFlow(tracker,flowObj) } as State,
                }) 
              default:
                this.setState({
                  currentState: {type: defaultFlowConfig.flow.toUpperCase(), result: pinFlow(tracker,defaultFlowConfig) } as State,
                })
                break;
            }
          }else{
            this.setState({
              currentState: {type: defaultFlowConfig.flow.toUpperCase(), result: pinFlow(tracker,defaultFlowConfig) } as State,
            })
          }
        }catch(err){
          this.setState({
            currentState: {type: defaultFlowConfig.flow.toUpperCase(), result: pinFlow(tracker,defaultFlowConfig) } as State,
          })
          throw err
        }
      }
    }

    async pinOneClickMoDetectOperatorByIpOrMsisdn(){
      if (typeof window != "undefined") {
        try{
          const operator = await tryGetIPRangeName(window.pac_analytics.visitor);
          const flowObj = determineFlowByOperator(operator ? operator.toUpperCase() : "");
          this.setState({
            strategyConfig: flowObj
          })
          if(operator !== null && flowObj.hasOwnProperty("flow")){  
            switch (flowObj.flow.toUpperCase()) {
              case "ONECLICK":
                this.setState({
                  currentState: {type: "ONECLICK", result: oneClickFlow(tracker,flowObj, (!!extraStrategyParams &&  !!extraStrategyParams["oneClick"]) ? extraStrategyParams["oneClick"] : null) } as State,
                })   
                break;
              case "MO":
                this.setState({
                  currentState: {type: "MO", result: moFlow(tracker,flowObj) } as State,
                }) 
                break;
              default:
                this.setState({
                  currentState: {type: "MSISDNEntry", result: RDS.NothingYet<MSISDNEntryFailure, MSISDNEntrySuccess>()} as State,
                })
                break;
            }
          }else{
            this.setState({
              currentState: {type: "MSISDNEntry", result: RDS.NothingYet<MSISDNEntryFailure, MSISDNEntrySuccess>()} as State,
            })
          }
        }catch(err){
          this.setState({
            currentState: {type: "MSISDNEntry", result: RDS.NothingYet<MSISDNEntryFailure, MSISDNEntrySuccess>()} as State,
          })
          throw err
        }
      }
    }


    async askOperator(){
      const operators = await getOperators();
      if(operators.length === 1){
        this.state.actions.selectOperator(operators[0]);
      }else{
        this.setState({
          currentState: {type: "OPERATORSelect", result:RDS.Success({operators})} as State,
        })
      } 
    }

    async componentWillMount(){      
      //isMocked for mocked flow states for design
      if(!isMocked){
        switch (configs.strategy) {
          case "pin-mo-ask-operator":
            this.askOperator();   
            break;
          case "pin-click2sms-ask-operator":
            this.askOperator(); 
            break;
          case "pin-click2sms-mo-ask-operator":
            this.askOperator();   
            break;
          case "click2sms-mo-ask-operator":
            this.askOperator(); 
            break;
          case "pin-mo-ask-mobile-number":
            this.setState({
              currentState: {type: "MSISDNEntry", result: RDS.NothingYet<MSISDNEntryFailure, MSISDNEntrySuccess>()} as State,
            })
            break;
          case "pin-mo-redir-ask-mobile-number":
            this.setState({
              currentState: {type: "MSISDNEntry", result: RDS.NothingYet<MSISDNEntryFailure, MSISDNEntrySuccess>()} as State,
            })
            break;
          case "pin-one-click-detect-operator-by-ip":
            this.pinOneClickDetectOperatorByIp();
            break;
          case "one-click-mo-pin-detect-operator-by-ip-or-msisdn":
            this.pinOneClickMoDetectOperatorByIpOrMsisdn();
            break;
          case "mo-redir-one-click-detect-operator-by-ip":
            this.moRedirOneClickDetectOperatorByIp();
            break;
          case "pin-click2sms-ask-mobile-number":
            this.setState({
              currentState: {type: "MSISDNEntry", result: RDS.NothingYet<MSISDNEntryFailure, MSISDNEntrySuccess>()} as State,
            })
            break;
          case "click2sms-mo-ask-mobile-number":
            this.setState({
              currentState: {type: "MSISDNEntry", result: RDS.NothingYet<MSISDNEntryFailure, MSISDNEntrySuccess>()} as State,
            })
            break;
        }

      }


    }
    render() {
      return (
        <Comp {...this.props} currentState={this.state.currentState} actions={this.state.actions} />
      );
    }
  };

  HOC.prototype['_client'] = "super-strategy";

  return HOC;
}