// import { submitMSISDNWithConfig as tallymanSubmitMSISDNWithConfig} from "../tallyman.v1-mo/main"
import { IFlow } from "../super-strategy/HOC";

export type IConfig = {
  offer?: number;
  slug?: string;
  country?: string;
  host?:string;
  device?: "smart" | "feature";
  queryString?: Array<[string, string]>;
}

export interface ILinkExtraParams {
  [key: string]: string
}

export type INextAction = {
  flow:string;
  url:string;
  type:"get";
  show_msisdn_prompt:boolean;
  show_operator_list:boolean;

}
export type IIDentifyUserResult = {
  success:true;
  rockman_id: string;
  redirect_url:string;
} |
{
  success: false;
  rockman_id: string;
  message: string;
  next_action:INextAction;
}

export type ISubscribeResult = {
  success:true;
  rockman_id: string;
  product_url:string;
} |
{
  success: false;
  rockman_id: string;
  message: string;
}

export type IMSISDNSubmissionResult = {
  success: true
  rockman_id: string
  redirect_url?: string
} | {
  success: false
  rockman_id: string
  message: string
  redirect_url?: string
 }
// &msisdn%5B0%5D=506541284
export const additionalQueryString = `msisdnSubmitted=Y&legalCheckbox=Y&incentivizedCheckbox=Y&op_confirmCheckbox=N`


const getConfig = ():IConfig=>{
  try {
    const configs:IConfig = require("../../../../../config.json");
    return configs;
  } catch (ex) {
    throw "config.json not found";
  }
};

export default async function submitMSISDN(window: Window, maybeConfig: IConfig, internationalMSISDN: string, bupperMSISDN: string, strategy?:IFlow, extraParams?: {[key: string]: string}): Promise<string> {
 console.table([{internationalMSISDN, bupperMSISDN}])
  const config: IConfig = strategy ? {
    offer:window.pac_analytics.visitor.offer,
    ...strategy.flowConfig
  } : { offer:window.pac_analytics.visitor.offer, ...getConfig(), ...(maybeConfig||{})}
  return submitMSISDNWithConfig(window, config, internationalMSISDN, extraParams)
}

// export async function submitMSISDNWithConfig(window: Window, config: IConfig, internationalMSISDN: string, bupperMSISDN: string, extraParams?: { [key: string]: string }): Promise<string> {

//   const { slug, country, device, offer, host } = config
//   const search = window.location.search.length > 1 ? ('&' + window.location.search.substr(1)) : ''
//   const extraParamsQs = !extraParams ? '' : '&' + Object.keys(extraParams).map(k => `${k}=${extraParams[k]}`).join('&')
//   const rockmanId = window.pac_analytics.visitor.rockmanId

//   //await tallymanSubmitMSISDNWithConfig(window, config, internationalMSISDN, extraParams);
//   return `https://${host}/${country}/${slug}?offer=${offer}&device=${device}&rockman_id=${rockmanId}&msisdn%5B0%5D=${bupperMSISDN}&${additionalQueryString}${extraParamsQs}${search}`
// }

export async function submitMSISDNWithConfig(window: Window, config: IConfig, internationalMSISDN: string, extraParams?: { [key: string]: string }): Promise<string> {
  const { slug, country, device, offer, host } = config
  const bupperizeCountry = c => c == 'gb' ? 'uk' : c;
  const search = window.location.search.substr(1) || ''
  const extraParamsQs = !extraParams ? '' : '&' + Object.keys(extraParams).map(k => `${k}=${extraParams[k]}`).join('&')
  const msisdn = internationalMSISDN.match(/\d+/gi).join('')
  const rockmanId = window.pac_analytics.visitor.rockmanId
  const result: IMSISDNSubmissionResult = await fetch(`https://${host}/tallyman/v1/?action=redirect&slug=${slug}&country=${country}&msisdn=${msisdn}&device=${device}&offerId=${offer}&rockman_id=${rockmanId}&page=${window.location.href}&pixel_url=${encodeURIComponent(`http://c1.ouisys.com/pixels?xcid=${window.location.pathname.replace("/", "")}&xaid=${window.pac_analytics.visitor.xaid}&country=${bupperizeCountry(country)}`)}${search}${extraParamsQs}`).then(x => x.json())

  if (false === result.success) {
    const type = result.message == 'ALREADY SUBSCRIBED' ? "AlreadySubscribed" : "InvalidMSISDN";
    const error = new Error(`${type}:\n${result.message}`)
    error['type'] = type;
    console.error(error)
    throw error
  } else {
    return result.redirect_url
  }
}




const getOrOverrideConfig = (maybeConfig: IConfig, strategy?:IFlow) : IConfig => {
  const config: IConfig = strategy ? {
    ...strategy.flowConfig
  } : !maybeConfig ? {} : maybeConfig
  const { host, country, slug, device, queryString } = getConfig()
  return { host: config.host || host, slug: config.slug || slug, country: config.country || country, device: config.device || device, queryString: (config.queryString || []).concat(queryString || []), }
} 

const fetchJsonp: <T>(_url:string)=>Promise<T> = (_url:string)=>{
  let url = _url;
  let ouisys_callback_function = "ouisys_callback_function" + Math.floor(Math.random() * 10000)
  return new Promise((resolve, reject) => {
    
    window[ouisys_callback_function] = (arg)=>{
      resolve(arg);
    }

    url += (url.indexOf('?') === -1) ? '?' : '&';
    url += `jsonp=${ouisys_callback_function}`
    const script = document.createElement("script");
    script.src = url;
    script.id = ouisys_callback_function;
    document.body.appendChild(script)
    
    // Caught if got 404/500
    script.onerror = () => {
      reject(new Error(`JSONP request to ${_url} failed`));
    };
  });
}

export const getRedirectUrl = async(maybeConfig: IConfig, extraParams?:ILinkExtraParams, strategy?:IFlow): 
  Promise<string> => {
  if (typeof window == "undefined") {
    throw "javascript: console.error('SSR'); void 6"
  } else {
    const { host, country, slug, queryString, device } = getOrOverrideConfig(maybeConfig, strategy)
    const bupperizeCountry = c => c == 'gb' ? 'uk' : c;
    const offer = window.pac_analytics.visitor.offer
    const rockmanId = window.pac_analytics.visitor.rockmanId
    const s = window.location.search.substr(1)
    const search = (!!s && s.length > 0 ? [s] : []).concat((queryString || []).map(([k, v]) => `${k}=${v}`)).join('&')
    const extraParamsQs = !extraParams ? '' : '&' + Object.keys(extraParams).map(k => `${k}=${extraParams[k]}`).join('&')
    
    const url = `https://${host}/tallyman/v1/?action=identify-user&country=${bupperizeCountry(country)}&slug=${slug}&device=${device}&offerId=${offer}&page=${window.location.href}&pixel_url=${encodeURIComponent(`http://c1.ouisys.com/pixels?xcid=${window.location.pathname.replace("/", "")}&xaid=${window.pac_analytics.visitor.xaid}&country=${bupperizeCountry(country)}`)}&rockman_id=${rockmanId}${extraParamsQs}&${search}`
    
    const result = await fetchJsonp<IIDentifyUserResult>(url); 
    if(false === result.success){
      const identifyError = new Error(`Error in getRedirectUrl() identify-user action:\n${result.message}`)
      identifyError['type'] = "NotOn3g";
      identifyError['next_action'] = result.next_action;
      throw identifyError
    }else{
      return result.redirect_url
    }
  }
}