import * as React from "react";
import * as RDS from "../../common-types/RemoteDataState";
import { ITracker } from "../../pacman/record";
import load, { IKeywordShortcode, IConfig, checkSubscription } from "./main";
import { MOLink, formatSMSLink } from "../common-components/MOLink";
import { IFlow } from "../super-strategy/HOC";


type IWithMultiKeywordShortcode  = IKeywordShortcode & {
  type?:string,
  data?:Array<IKeywordShortcode>
}
export type State = RDS.RemoteDataState<string, IWithMultiKeywordShortcode>

export const initialState : State = RDS.NothingYet() as State
export const mockedSuccessState : State = RDS.Success<string, IKeywordShortcode>({ keyword: 'TEST OK', shortcode: '666' })
export const mockedFailureState : State = RDS.Failure<string, IKeywordShortcode>("Mocked Network Error") as State
export const mockedLoadingState : State = RDS.Loading<string, IKeywordShortcode>()

export { IKeywordShortcode, formatSMSLink }

export type HOCProps = {
  currentState: State,
  actions?: {
    moLink:ACompType
    onSetKeyword?: (keyword: string) => void
    onSetKeywordAndShortcode?: (keyword: string, shortcode) => void
  },
  MOLink: ACompType,
  strategyConfig?:IFlow,
}

type ACompType = 
    React.ComponentType<{children: React.ReactNode} & React.HTMLAttributes<HTMLAnchorElement> & {keyword?: string, shortcode?: string, operators?:string[]}>
export default <P extends {}>(tracker: ITracker, Comp: React.ComponentType<P & HOCProps>, maybeConfig?: IConfig, strategy?:IFlow) => (
  initialState: State
) : React.ComponentType =>{
  const HOC = class HOC extends React.PureComponent<P> {
    state = {
      currentState: initialState,
      keyword: null as string,
      shortcode: null as string
    }

    actions = {
      moLink: null,
      onSetKeyword: (keyword: string) => {
        this.setState({ keyword })
        load(window, maybeConfig, keyword, null, strategy).then(
          x => this.setState({ currentState: RDS.Success(x) as State })
        ).catch(
          e => this.setState({ currentState: RDS.Failure(e.toString()) as State })
        )
      },
      onSetKeywordAndShortcode: (keyword: string, shortcode: string) => {
        this.setState({ keyword, shortcode })
        load(window, maybeConfig, keyword, shortcode, strategy).then(
          x => this.setState({ currentState: RDS.Success(x) as State })
        ).catch(
          e => this.setState({ currentState: RDS.Failure(e.toString()) as State })
        )
      }
    }

    componentDidMount() {
      if(!!maybeConfig && maybeConfig.tag == "keywordAndShortCode") {
        this.setState({ currentState: RDS.Success({
          type:"SingleSubmissionResult",
          ...maybeConfig
        }) as State })
      }else if((!!maybeConfig && maybeConfig.tag != "keyword") || !maybeConfig){
        this.setState({currentState: RDS.Loading() as State})
        load(window, maybeConfig, null,null,  strategy).then(
          x => this.setState({currentState: RDS.Success(x) as State})
        ).catch(
          e => this.setState({currentState: RDS.Failure(e.toString()) as State})
        )
        console.log("currentState", this.state.currentState)
      }
    }

    render() {
      const self = this;
      const moLink = RDS.match<string, IWithMultiKeywordShortcode, ACompType>({
        nothingYet: () => ({children, ...props} ) => <a data-state="nothingYet" onClick={() => console.info("nothingYet")} href="javascript: void(0)" {...props}>{children}</a>,
        loading: () => ({children, ...props}) => <a data-state="loading" onClick={() => console.info("loading")} href="javascript: void(0)" {...props}>{children}</a> ,
        failure: (error) => ({children, ...props}) => <a data-state="failure" onClick={() => console.error(error)} href="javascript: void(0)" {...props}>{children}</a> ,
        success: (result) => ({children, keyword, shortcode, operators, ...props}) => {

          if(result.type == "SingleSubmissionResult"){
            const keywordAndShortcode1 = {
              keyword: keyword || result.keyword, 
              shortcode: shortcode || result.shortcode
            }
            return(
              <MOLink onClick={() => {
                tracker.advancedInFlow("click2sms", "click", keywordAndShortcode1)
                try{
                  checkSubscription(window, maybeConfig, strategy)
                  .then((product_url)=>{
  
                    tracker.advancedInFlow('click2sms', 'sale', { keywordAndShortcode1 });
                    setTimeout(() => {
                      window.location.href = product_url
                    }, 1000);
                  })
                }catch(err){
                  throw err;  
                }
              }} 
                keywordAndShortcode={keywordAndShortcode1} {...props}
              >{children}</MOLink>
            ) 
          }else{
            //console.log("result", result)
            return(
              <React.Fragment>
                {
                  result.data && result.data.map((obj, index)=>{
                    const keywordAndShortcode1 = {
                      keyword: keyword || obj.keyword, 
                      shortcode: shortcode || obj.shortcode
                    }
                  if(!!operators && operators.length > 0 && operators.includes(obj.operator)){
                    return (
                      <MOLink
                        key={index}
                        onClick={() => tracker.advancedInFlow("click2sms", "click", keywordAndShortcode1)} 
                        keywordAndShortcode={keywordAndShortcode1} {...props}
                      >{children}</MOLink>
                    )
                  }else if(!operators){
                    return (
                      <MOLink
                        key={index}
                        onClick={() => tracker.advancedInFlow("click2sms", "click", keywordAndShortcode1)} 
                        keywordAndShortcode={keywordAndShortcode1} {...props}
                      >{children}</MOLink>
                    )
                  }
           
                  })
                }
              </React.Fragment>
            )
          }
          
        }
      })(this.state.currentState)

      const moLinks = RDS.match<string, IWithMultiKeywordShortcode, ACompType>({
        nothingYet: () => ({children, ...props} ) => <a data-state="nothingYet" onClick={() => console.info("nothingYet")} href="javascript: void(0)" {...props}>{children}</a>,
        loading: () => ({children, ...props}) => <a data-state="loading" onClick={() => console.info("loading")} href="javascript: void(0)" {...props}>{children}</a> ,
        failure: (error) => ({children, ...props}) => <a data-state="failure" onClick={() => console.error(error)} href="javascript: void(0)" {...props}>{children}</a> ,
        success: (result) => ({children, keyword, shortcode, ...props}) => {
            return(
              <div>
                {
                  result.data.map((obj, index)=>{
                    const keywordAndShortcode1 = {
                      keyword: keyword || obj.keyword, 
                      shortcode: shortcode || obj.shortcode
                    }
                    
                  return (
                    <MOLink
                      onClick={() => {
                        tracker.advancedInFlow("click2sms", "click", keywordAndShortcode1)
                        try{
                          checkSubscription(window, maybeConfig, strategy)
                          .then((product_url)=>{
          
                            tracker.advancedInFlow('click2sms', 'sale', { keywordAndShortcode1 });
                            setTimeout(() => {
                              window.location.href = product_url
                            }, 1000);
                          })
                        }catch(err){
                          throw err;  
                        }
                      }} 
                      keywordAndShortcode={keywordAndShortcode1} {...props}
                    >{children}</MOLink>
                    )
                  })
                }
              </div>
            )

        }
      })(this.state.currentState)
      self.actions.moLink = moLink;
      return <Comp 
                {...this.props} 
                currentState={this.state.currentState} 
                actions={this.actions}
                MOLink={self.actions.moLink}
                strategyConfig={strategy ? strategy : undefined}
              />
    }
  }
  HOC.prototype['_client'] = "tallyman.v1-click2sms";

  return HOC;
}