import { IFlow } from "../super-strategy/HOC";
export type IConfig = {
  offer: number,
  slug?: string;
  country?: string;
  host?:string;
  device?: "smart" | "feature";
}
  

export type IKeywordShortcode = {
  keyword : string,
  shortcode : string
}

export type IMSISDNSubmissionResult = {
  success: true
  rockman_id: string,
  keyword: string,
  shortcode: string
} | {
  success: false
  rockman_id: string
  message: string
}
export type ICheckSubscriptionResult = {
  success: false;
  rockman_id: string;
  message: string;
} | {
  success: true;
  msisdn: string;
  product_url?: string;
  rockman_id: string;
}
const getConfig = ():IConfig=>{
  try {
    const configs:IConfig = require("../../../../../config.json");
    return configs;
  } catch (ex) {
    throw "config.json not found";
  }
};

export default async function submitMSISDN(window: Window, maybeConfig: IConfig, internationalMSISDN: string, strategy?:IFlow, extraParams?: {[key: string]: string}): Promise<IKeywordShortcode> {
  const config: IConfig = strategy ? {
    offer:window.pac_analytics.visitor.offer,
    ...strategy.flowConfig
  } : { offer:window.pac_analytics.visitor.offer, ...getConfig(), ...(maybeConfig||{})}
  return submitMSISDNWithConfig(window, config, internationalMSISDN, extraParams)
}

export async function submitMSISDNWithConfig(window: Window, config: IConfig, internationalMSISDN: string, extraParams?: { [key: string]: string }): Promise<IKeywordShortcode> {
  const { slug, country, device, offer, host } = config
  const search = window.location.search.substr(1) || ''
  const extraParamsQs = !extraParams ? '' : '&' + Object.keys(extraParams).map(k => `${k}=${extraParams[k]}`).join('&')
  const msisdn = internationalMSISDN.match(/\d+/gi).join('')
  const rockmanId = window.pac_analytics.visitor.rockmanId
  const result: IMSISDNSubmissionResult = await fetch(`https://${host}/tallyman/v1/?action=trigger-message&slug=${slug}&country=${country}&msisdn=${msisdn}&device=${device}&offerId=${offer}&rockman_id=${rockmanId}&${search}${extraParamsQs}`).then(x => x.json())

  if (false === result.success) {
    const type = result.message == 'ALREADY SUBSCRIBED' ? "AlreadySubscribed" : "InvalidMSISDN";
    const error = new Error(`${type}:\n${result.message}`)
    error['type'] = type;
    console.error(error)
    throw error
  } else {
    return result
  }
}

const wait = (ms: number) =>
  new Promise(resolve => setTimeout(() => resolve(), ms));

const loop = async <I extends {}, R extends {}>(
  f: () => Promise<R>,
  max: number,
  stop: (r: R) => boolean,
  i: number
): Promise<R> => {
  const r = await f();
  if (stop(r) || i >= max) {
    return r;
  } else {
    await wait(10000);
    return loop(f, max, stop, i + 1);
  }
};

export async function checkSubscription(window: Window, maybeConfig: IConfig, internationalMSISDN: string, strategy?:IFlow, extraParams?: {[key: string]: string}): Promise<string> {
  const config: IConfig = strategy ? {
    offer:window.pac_analytics.visitor.offer,
    ...strategy.flowConfig
  } : { offer:window.pac_analytics.visitor.offer, ...getConfig(), ...(maybeConfig||{})};
  const { host } = config

  const rockmanId = window.pac_analytics.visitor.rockmanId
  const check : () => Promise<ICheckSubscriptionResult> = () => fetch(`https://${host}/tallyman/v1/?action=check-subscription&rockman_id=${rockmanId}`)
  .then(x => x.json())

  const checkResult: ICheckSubscriptionResult = await loop(
    check,
    60,
    r => r.success,
    0
  )

  if(true === checkResult.success) {
    window.location.href = checkResult.product_url;
    return checkResult.product_url || null
  } else {
    const error = new Error(`${"SubscriptionFailed"}:\n${checkResult.message}`)
    error['type'] = "SubscriptionFailed";
    throw error
  }
}
