import { submitMSISDNWithConfig as tallymanSubmitMSISDNWithConfig} from "../tallyman.v1-mo/main"
import { IFlow } from "../super-strategy/HOC";
import tryGetIPRangeName from "../../ips/tryGetIPRangeName";

export type IConfig = {
  offer: number,
  slug?: string;
  country?: string;
  host?:string;
  device?: "smart" | "feature";
}
export type IMSISDNSubmissionResult = {
  success: true
  rockman_id: string,
  redirect_url: string
} | {
  success: false
  rockman_id: string
  message: string
}

// &msisdn%5B0%5D=506541284
export const additionalQueryString = `msisdnSubmitted=Y&legalCheckbox=Y&incentivizedCheckbox=Y&op_confirmCheckbox=N`

const getConfig = ():IConfig=>{
  try {
    const configs:IConfig = require("../../../../../config.json");
    return configs;
  } catch (ex) {
    throw "config.json not found";
  }
};

export default async function submitMSISDN(window: Window, maybeConfig: IConfig, internationalMSISDN: string, bupperMSISDN: string, strategy?:IFlow, extraParams?: {[key: string]: string}): Promise<string> {
 console.table([{internationalMSISDN, bupperMSISDN}])
  const config: IConfig = strategy ? {
    offer:window.pac_analytics.visitor.offer,
    ...strategy.flowConfig
  } : { offer:window.pac_analytics.visitor.offer, ...getConfig(), ...(maybeConfig||{})}
  return submitMSISDNWithConfig(window, config, internationalMSISDN, bupperMSISDN, extraParams)
}

export async function submitMSISDNWithConfig(window: Window, config: IConfig, internationalMSISDN: string, bupperMSISDN: string, extraParams?: { [key: string]: string }): Promise<string> {
 
  const { slug, country, device, offer, host } = config
  const search = window.location.search.substr(1) || ''
  const extraParamsQs = !extraParams ? '' : '&' + Object.keys(extraParams).map(k => `${k}=${extraParams[k]}`).join('&')
  const msisdn = internationalMSISDN.match(/\d+/gi).join('')
  const visitor = window.pac_analytics.visitor;
  const rockmanId = visitor.rockmanId;

  const ip_range_name = country == "ci" ? await tryGetIPRangeName(visitor) : null;

  const result: IMSISDNSubmissionResult = await fetch(`https://${host}/tallyman/v1/?action=redirect&slug=${slug}&country=${country}&msisdn=${msisdn}&device=${device}&offerId=${offer}&rockman_id=${rockmanId}&network_type=${!ip_range_name ? "wifi":""}&${search}${extraParamsQs}`).then(x => x.json())

  if (false === result.success) {
    const type = result.message == 'ALREADY SUBSCRIBED' ? "AlreadySubscribed" : "InvalidMSISDN";
    const error = new Error(`${type}:\n${result.message}`)
    error['type'] = type;
    console.error(error)
    throw error
  } else {
    return result.redirect_url
    
  }
}

