import { IFlow } from "../super-strategy/HOC";

export type IClickActionTypes = "he" | "redirect" | "identify-user";
export type IConfig = {
  host?: string, country?: string, slug?: string, queryString?: Array<[string, string]>, device?:string, apiAction?:IClickActionTypes,
}

export interface ILinkExtraParams {
  [key: string]: string
}

export type IIDentifyUserResult = {
  success:true;
  rockman_id: string;
  subscription_url?:string;
  redirect_url?:string;
} |
{
  success: false;
  rockman_id: string;
  message: string;
}

export type IHEResult = {
  success:true;
  rockman_id: string;
  redirectUrlWithReplacedUniqid?:string;
  source:string;
  oldUniqid: string;
	newUniqid: string;
} |
{
  success: false;
  rockman_id: string;
  message: string;
}

export type ISubscribeResult = {
  success:true;
  rockman_id: string;
  product_url:string;
} |
{
  success: false;
  rockman_id: string;
  message: string;
}



const getConfig = ():IConfig=>{
  try {
    const configs:IConfig = require("../../../../../config.json");
    return configs;
  } catch (ex) {
    throw "config.json not found";
  }
};

const getOrOverrideConfig = (maybeConfig: IConfig, strategy?:IFlow) : IConfig => {
  const config: IConfig = strategy ? {
    ...strategy.flowConfig
  } : !maybeConfig ? {} : maybeConfig
  const { host, country, slug, queryString, apiAction } = getConfig()

  return { host: config.host || host, slug: config.slug || slug, country: config.country || country, queryString: (config.queryString || []).concat(queryString || []), apiAction:config.apiAction || apiAction}
} 

const fetchJsonp: <T>(_url:string)=>Promise<T> = (_url:string)=>{
  let url = _url;
  let ouisys_callback_function = "ouisys_callback_function" + Math.floor(Math.random() * 10000)
  return new Promise((resolve, reject) => {
    
    window[ouisys_callback_function] = (arg)=>{
      resolve(arg);
    }

    url += (url.indexOf('?') === -1) ? '?' : '&';
    url += `jsonp=${ouisys_callback_function}`
    const script = document.createElement("script");
    script.src = url;
    script.id = ouisys_callback_function;
    document.body.appendChild(script)
    
    // Caught if got 404/500
    script.onerror = () => {
      reject(new Error(`JSONP request to ${_url} failed`));
    };
  });
}

const loadScriptSrc = (src)=>{
  var script = document.createElement('script');
  script.type = 'text/javascript';
  script.src = src
  document.body.appendChild(script);
  console.log('loadScript');
}

const loadScriptInnerHtml = (innerHTML)=>{
  var script = document.createElement('script');
  script.type = 'text/javascript';
  script.innerHTML = innerHTML
  document.body.appendChild(script);
  console.log('loadScript');
}
const getRedirectUrl = async(maybeConfig: IConfig, extraParams?:ILinkExtraParams, strategy?:IFlow): 
  Promise<{
    subscription_url?:string;
    redirect_url?:string;
    action?:()=>Promise<string>;
  }> => {
  if (typeof window == "undefined") {
    throw "javascript: console.error('SSR'); void 6"
  } else {
    const { host, country, slug, queryString, device, apiAction } = getOrOverrideConfig(maybeConfig, strategy);
    
    const bupperizeCountry = c => c == 'gb' ? 'uk' : c;
    const offer = window.pac_analytics.visitor.offer
    const rockmanId = window.pac_analytics.visitor.rockmanId
    const s = window.location.search.substr(1)
    const search = (!!s && s.length > 0 ? [s] : []).concat((queryString || []).map(([k, v]) => `${k}=${v}`)).join('&')
    const extraParamsQs = !extraParams ? '' : '&' + Object.keys(extraParams).map(k => `${k}=${extraParams[k]}`).join('&')

    console.log("apiAction", apiAction)


    if(apiAction == "identify-user"){

      const url = `https://${host}/tallyman/v1/?action=identify-user&country=${bupperizeCountry(country)}&slug=${slug}&device=${"smart"}&offerId=${offer}&rockman_id=${rockmanId}${extraParamsQs}&${search}`
      const result = await fetchJsonp<IIDentifyUserResult>(url); 
      if(false === result.success){
        const identifyError = new Error(`Error in getRedirectUrl() identify-user action:\n${result.message}`)
        identifyError['type'] = "NotOn3g";
        throw identifyError
      }else{
        return {subscription_url:result.subscription_url || result.redirect_url, action:()=>subscribe(result.subscription_url)}
      }

    }else if(apiAction == "he"){
      const url = `https://${host}/tallyman/v1/?action=he&country=${bupperizeCountry(country)}&slug=${slug}&device=${"smart"}&offerId=${offer}&rockman_id=${rockmanId}${extraParamsQs}&${search}`
      const result =  await fetch(url).then(x => x.json());
      if(false === result.success){
        const identifyError = new Error(`Error in getRedirectUrl() identify-user action:\n${result.message}`)
        identifyError['type'] = "NotOn3g";
        throw identifyError
      }else{
        loadScriptInnerHtml(result.source)
        return {redirect_url:result.redirectUrlWithReplacedUniqid}
      }
    }else{
      const url = `https://${host}/tallyman/v1/?action=redirect&country=${bupperizeCountry(country)}&slug=${slug}&device=${"smart"}&offerId=${offer}&rockman_id=${rockmanId}${extraParamsQs}&${search}`
      const result = await fetch(url).then(x => x.json())
      
      if(false === result.success){
        const identifyError = new Error(`Error in getRedirectUrl() identify-user action:\n${result.message}`)
        identifyError['type'] = "NotOn3g";
        throw identifyError
      }else{
        return {redirect_url:result.redirect_url}
      }

    }
  }
}

const subscribe:(url:string)=>Promise<string> = async(url:string)=>{

  const result:ISubscribeResult =  await fetch(url).then((x)=>x.json())

  if(false === result.success){
    const subscribeError = new Error(`Error in subscribe() one-click-subscribe action:\n${result.message}`)
    subscribeError['type'] = "SubscriptionError";
    throw subscribeError
  }else{
    return result.product_url
  }

}
export default getRedirectUrl