import * as path from 'path'
import * as fs from 'fs'
import * as R from 'ramda'
import inquirer from 'inquirer';
import * as envfile from "envfile"
import axios from "axios";

type ClientTypes = 
  | "assrock-mo"
  | "assrock-pin"
  | "bupper-click2sms"
  | "bupper-one-click"
  | "tallyman.v1-pin"
  | "tola-mpesa"
  | "tallyman.v1-mo"
  | "tallyman.v1-mo-redir"
  | "tallyman.v1-one-click"
  | "tallyman.v1-click-redir";

export type IClickActionTypes = "he" | "redirect" | "identify-user";

const env = envfile.parseFileSync(path.resolve("", '.env'))
export type ConfigDetails = {
  host: string;
  slug?: string;
  handle?:string;
  country: string;
  device: "smart" | "feature";

  // if 'automaticallySubmitAllOperators' is set to true, the client tries to trigger pin for every operator
  automaticallySubmitAllOperators?: boolean;
  operators?: string[];
  apiAction?:IClickActionTypes;
}
export type Config = {
  name: string;
  service: string;
  config: ConfigDetails
};
export default async function (page_type : ClientTypes) {
  //const options = require(`../clients/${page_type}/config`).default as Config
  const result = await axios({
    url:'http://c1.ouisys.com/api/v1/get-config',
    //url:'http://localhost:3030/api/v1/get-page-legals',
    method: 'get',
    params: {
      client:page_type
    }
  });
  const data = result.data;
  //const dataKeys = Object.keys(options);
  // const countries = R.pipe(
  //   R.map((o: any) => options[o].config.country as string),
  //   R.uniq,
  //   R.sortBy(x => x),
  //   xs => (!!env.country ? [env.country] : []).concat(xs)
  // )(dataKeys)

  const countries = R.pipe(
      R.map((o: any) => o.country as string),
      R.uniq,
      R.sortBy(x => x),
      xs => (!!env.country ? [env.country] : []).concat(xs)
    )(data)
  
  
  console.log(`You are configuring '${page_type}' template.`)
  // first choose a country
  const {country} = await inquirer.prompt([
    {
      type: "list",
      name: "country",
      message: `First select a country. ${env.country ? `Current country: ${env.country}` : ''}`,
      choices: countries.map(c => ({
        name: c, value: c
      }))
    }
  ])

  const scenarios = R.pipe(
    R.filter((x: any) => x.country == country),
  )(data)


  const {scenarioService} = await inquirer.prompt([
    {
      type: "list",
      name: "scenarioService",
      message: `Now select a scenario for ${country}`,
      choices: scenarios.map(s => ({
        name: `(SCENARIO:${s.scenario} & SERVICE:${s.service}) - ${s.host}/${s.country}/${s.scenario}`
        , value: s
      }))
    }
  ])

  const scenario = scenarioService.scenario;
  const service= scenarioService.service;

  const updatedEnvContent = envfile.stringifySync({
    ...env,
    country,
    scenario,
    service
  })

  fs.writeFileSync(path.resolve("", ".env"), updatedEnvContent, "utf8")
  fs.writeFileSync(path.resolve("", "config.json"), JSON.stringify({
    automaticallySubmitAllOperators:(scenarioService.automatically_submit_all_operators !== null ) ? scenarioService.automatically_submit_all_operators : false,
    country,
    device:scenarioService.device,
    host:scenarioService.host,
    slug:scenario,
    operators:scenarioService.operators ? scenarioService.operators : [],
    service,
    queryString:scenarioService.query_string ? scenarioService.query_string : undefined,
    apiAction:(scenarioService.api_action !== null ) ? scenarioService.api_action : undefined,
  }, null, 2), "utf8")

  console.table([{country, scenario, service}])
}
