import * as path from 'path'
import * as fs from 'fs'
import * as R from 'ramda'
import inquirer from 'inquirer';
import * as envfile from "envfile";
import axios from "axios";

import { StrategyConfig } from '../clients/super-strategy/HOC';

type StrategyTypes = 
  | "super-strategy"

const env = envfile.parseFileSync(path.resolve("", '.env'))

const extraConfigInfo = {
  pinScenario:null,
  pinService:null,
  oneClickScenario:null,
  oneClickService:null,
  moScenario:null,
  moService:null,
  click2smsScenario:null,
  click2smsService:null,
  moRedirScenario:null,
  moRedirService:null
};

export default async function (page_type : StrategyTypes) {
  // const options = require(`../clients/${page_type}/config`).default as Array<{
  //     name: string,
  //     config: {
  //       host: string,
  //       country: string,
  //       handle?: string,
  //       slug?:string,
  //     }
  //   }>
  const result = await axios({
    url:'http://c1.ouisys.com/api/v1/get-strategies-config',
    //url:'http://localhost:3030/api/v1/get-page-legals',
    method: 'get'
  });
    const data = result.data;
    
  const countries = R.pipe(
    R.map((o: any) => o.country as string),
    R.uniq,
    R.sortBy(x => x),
    xs => (!!env.country ? [env.country] : []).concat(xs)
  )(data)

  
  console.log(`You are configuring '${page_type}' template.`)
  // first choose a country
  const {country} = await inquirer.prompt([
    {
      type: "list",
      name: "country",
      message: `First select a country. ${env.country ? `Current country: ${env.country}` : ''}`,
      choices: countries.map(c => ({
        name: c, value: c
      }))
    }
  ])

  const stategies = R.pipe(
    R.filter((x: any) => x.country == country),
    R.map((o: any) => o.strategy as string),
    R.uniq,
    R.sortBy(x => x),
  )(data);


  const { strategy } = await inquirer.prompt([
    {
      type: "list",
      name: "strategy",
      message: `Now select a strategy for ${country}`,
      choices: stategies.map((s:string) => ({
        name: s,
        value:s
      }))
    }
  ]);

  const strategyConfigs = R.pipe(
    R.filter((x: any) => x.country == country && x.strategy == strategy),
    R.map((o: any) => o)
  )(data);




   
  const configString = (s)=>{
    const operators = s.scenario_config.operators;
    const operatorsArray = Object.keys(operators);
    return operatorsArray.map((o, i)=>{
      extraConfigInfo[`${operators[o].flow}Scenario`] = operators[o].flowConfig.slug;
      extraConfigInfo[`${operators[o].flow}Service`] = operators[o].flowConfig.service;
      return(
        `${(i != 0) ? "| " : ""}OPERATOR:${o} FLOW: ${operators[o].flow.toUpperCase()} => SCENARIO: ${operators[o].flowConfig.slug} SERVICE: ${operators[o].flowConfig.service}}`
      )
    })

  }
  const {scenariosConfig} = await inquirer.prompt([
    {
      type: "list",
      name: "scenariosConfig",
      message: `Now select scenarios for the strategy ${strategy}`,
      choices: strategyConfigs.map(s => ({
        name: `${strategy} [${configString(s)}]`,
        value:s
      }))
    }
  ])



  const newExtraConfigObj = {};
  const extraConfigKeyArr = Object.keys(extraConfigInfo);
  extraConfigKeyArr.filter((k)=>extraConfigInfo[k] !== null).map((e)=>newExtraConfigObj[e] = extraConfigInfo[e]);

  const defaultFlow:StrategyConfig = scenariosConfig.scenario_config;

  

  const { operators } = scenariosConfig.scenario_config;
  const operatorsArr = Object.values(operators);

  operatorsArr.map((obj:any)=>{
    const {flowConfig} = obj;
    newExtraConfigObj[`${obj.flow}Scenario`] = flowConfig.slug; 
    newExtraConfigObj[`${obj.flow}Service`] = flowConfig.service;
    
    newExtraConfigObj[`${defaultFlow.default.flow}Scenario`] = defaultFlow.default.flowConfig.slug; 
    newExtraConfigObj[`${defaultFlow.default.flow}Service`] = defaultFlow.default.flowConfig.service;


        
    newExtraConfigObj["defaultScenario"] = defaultFlow.default.flowConfig.slug; 
    newExtraConfigObj["defaultService"] = defaultFlow.default.flowConfig.service;
  })
  


  //clean env of scenarios we dont need
  extraConfigKeyArr.map((e)=>{
    console.log("EEE", e)
    
    delete env[e]})


  const updatedEnvContent = envfile.stringifySync({
    ...env,
    ...newExtraConfigObj,
    country,
    strategy:strategy,
    scenariosConfig:scenariosConfig.scenario_config_name
  })

  const configObj = {
    strategy,
    country,
    strategyConfigs:scenariosConfig.scenario_config
  };
  fs.writeFileSync(path.resolve("", "config.json"),JSON.stringify(configObj, null, 2), "utf8")
  fs.writeFileSync(path.resolve("", ".env"), updatedEnvContent, "utf8")
}




