
import inquirer from 'inquirer';
import * as fs from 'fs'
import path from "path";
import * as envfile from "envfile";
import { exec } from "child_process";
import * as util from "util";

const env = envfile.parseFileSync(path.resolve("", ".env"));
const execute = (command, callback)=>{
  exec(command, (error, stdout, stderr) => callback(stdout));
};
const execPromise = util.promisify(require('child_process').exec);

  
const createAbTest =  async  ()=>{

  console.log("⚠️ IMPORTANT: Please confirm or change the following before building your project!");

  console.log("Press <enter> to accept default or type in a new value and press <enter>");


  
  const { page_name } = await inquirer.prompt([
    {
      type: "input",
      name: "page_name",
      message: `A/B Test Page Name:`,
    }
  ]);

  const updatedEnvContent = envfile.stringifySync({
    ...env,
    page: page_name || env.page
  })

  const { stdout , stderr } = await execPromise(`git branch abtest/${page_name}`);

  if(stderr !== ""){
    console.log(`🛑 ERROR: Creating A/B Test Branch: ${JSON.stringify(stderr)}`);
    process.exit(-1);
    return
  }else{
    await execPromise(`git checkout abtest/${page_name}`);

    fs.writeFileSync(path.resolve("", ".env"), updatedEnvContent, "utf8")

    console.log("A/B Test Branch Created! 😊");
  }
}

createAbTest();