import path from "path";
import { writeFileSync } from "fs";
import axios from "axios";
import * as envfile from "envfile";
import  { exec } from "child_process";

const execute = (command, callback)=>{
  exec(command, (error, stdout, stderr) => callback(stdout));
};

const disclaimerPath = `./src/localization/translations/legals.json`;

const env = envfile.parseFileSync(path.resolve("", ".env"));

const { country, service, scenario } = env;


const writeDisclaimers = (data, isSuccess)=>{
  writeFileSync(disclaimerPath, JSON.stringify(data, null, 2), "utf8")
  if(isSuccess){
    console.log(`😇 Disclaimers generated here ${path.resolve("","src/localization/translations/legals.json")}`);
  }
}
const getLegals = ()=>{
  return axios({
    url:'https://c1.ouisys.com/api/v1/get-page-legals',
    //url:'http://localhost:3030/api/v1/get-page-legals',
    method: 'get',
    params: {
      country,
      service,
      scenario
    }
  }).then(function(response) {
    if(response.data.length > 0){
      writeDisclaimers(response.data, true)
    }else{
      writeDisclaimers([], false)
      console.log(` 😔 ERROR: Could not get any disclaimers for the given data. Please check your .env file`);
      console.table([{country, scenario, service}]);
    }
    return response.data;
  })
  .catch((err)=>{
    console.error(err);
  })
};


export default getLegals;