import path from "path";
import { writeFileSync } from "fs";
import axios from "axios";
import * as envfile from "envfile";
import  { exec } from "child_process";
import { async } from "q";

const execute = (command, callback)=>{
  exec(command, (error, stdout, stderr) => callback(stdout));
};

const disclaimerPath = `./src/localization/translations/legals.json`;

const env = envfile.parseFileSync(path.resolve("", ".env"));

const extraConfigInfo = {
  pinScenario:null,
  moScenario:null,
  oneClickScenario:null,
  pinService:null,
  moService:null,
  oneClickService:null,
};
const {
  country,
  pinScenario,
  moScenario,
  oneClickScenario,
  pinService,
  moService,
  oneClickService,
  click2smsScenario,
  click2smsService
 } = env;

const serviceScenarios = [{
  scenario:pinScenario,
  service:pinService,
  key:"pin"
},{
  scenario:moScenario,
  service:moService,
  key:"mo"
},{
  scenario:oneClickScenario,
  service:oneClickService,
  key:"oneClick"
},{
  scenario:click2smsScenario,
  service:click2smsService,
  key:"click2sms"
}];


const getLegals = async(scenario, service)=>{
  try{
    const response = await axios({
      url:'https://c1.ouisys.com/api/v1/get-page-legals',
      //url:'http://localhost:3030/api/v1/get-page-legals',
      method: 'get',
      params: {
        country,
        service,
        scenario
      }
    })
    return response.data;
  }catch(err){
    throw err
  }
};


const arrToSubmit = serviceScenarios.filter((obj)=>obj.scenario !== undefined && obj.service)
async function getAllLegals (){
  const response = await Promise.all(
    arrToSubmit.map(async(obj)=>{
      if(obj.scenario !== "undefined" && obj.service !== "undefined"){
        try{
          const legals =  await getLegals(obj.scenario, obj.service);
          if(legals.length > 0){
            //append flow to each object
            legals.map((l)=>{
              l.flow = obj.key
            })
          }else{
            throw `Can't get legals Scenario: ${obj.scenario} and Service:${obj.service}`
          }
        return legals
        }catch(err){
          throw err;
          
        }
        
      }
    })
  );
  return Array.prototype.concat.apply([], response);
  //return response.data;
};


const writeDisclaimers = async()=>{
  try{
    const data = await getAllLegals();

    if(data.length > 0 && !data.includes(null)){
      writeFileSync(disclaimerPath, JSON.stringify(data, null, 2), "utf8")
      console.log(`😇 Disclaimers generated here ${path.resolve("","src/localization/translations/legals.json")}`);
      return
    }else{
      writeFileSync(disclaimerPath, JSON.stringify([], null, 2), "utf8")
      console.log(` 😔 ERROR: Could not get any disclaimers for the given data. Please check your .env file`);
      return
    }
  }catch(err){
    throw err
  }
}

export default writeDisclaimers;

