
import inquirer from 'inquirer';
import * as fs from 'fs'
import path from "path";
import * as envfile from "envfile";
import { exec } from "child_process";
import * as util from "util";

const env = envfile.parseFileSync(path.resolve("", ".env"));
const execute = (command, callback)=>{
  exec(command, (error, stdout, stderr) => callback(stdout));
};
const execPromise = util.promisify(require('child_process').exec);

  
const preBuild =  async  ()=>{

  console.log("⚠️ IMPORTANT: Please confirm or change the following before building your project!");

  console.log("Press <enter> to accept default or type in a new value and press <enter>");

  const updateConfig = (extraItems)=>{
    const updatedEnvContent = envfile.stringifySync({
      ...env,
      client: client || env.client,
      title: title || env.title,
      country: country || env.country,
      page: page || env.page,
      ...extraItems
    })
    execute("basename -s .git `git config --get remote.origin.url`", async (repo_name)=>{
      const repoName = repo_name.trim();
      const myPage = page ? page : env.page;
      const { stdout , stderr } = await execPromise("git rev-parse --abbrev-ref HEAD");
      const branch = stdout.trim();
      console.log("BRANCH NAME", branch)
      
      console.log("repoName", repoName)
      console.log("PAGE", myPage)

      if(branch.includes("abtest/") && branch.split("/")[1] !== myPage){
        console.log(`🛑 ERROR: Page name: ${myPage} must match git branch basename: ${branch.split("/")[1]}`);
        process.exit(-1);
        return
      }else if(!branch.includes("abtest/") && myPage !== repoName && process.env.SKIPCHECKS != "true"){
        console.log(`🛑 ERROR: Page name: ${myPage} must match git repo name: ${repoName}`);
        process.exit(-1);
        return
      }else{
        fs.writeFileSync(path.resolve("", ".env"), updatedEnvContent, "utf8")
        console.log(".env updated! 😊");
        
        console.table([{
          client: client || env.client,
          title: title || env.title,
          country: country || env.country,
          page: page || env.page,
          ...extraItems
        }])

        console.log("Building project please wait...")
      }
      
    });
  }
  const { client } = await inquirer.prompt([
    {
      type: "input",
      name: "client",
      message: `Client:  ${env.client ? `(${env.client})` : ""}`,
    }
  ]);
  const { title } = await inquirer.prompt([
    {
      type: "input",
      name: "title",
      message: `Title:  ${env.title ? `(${env.title})` : ""}`
    }
  ]);
  const { country } = await inquirer.prompt([
    {
      type: "input",
      name: "country",
      message: `Country:  ${env.country ? `(${env.country})` : ""}`
    }
  ]);
  const { page } = await inquirer.prompt([
    {
      type: "input",
      name: "page",
      message: `Page:  ${env.page ? `(${env.page})` : ""}`
    }
  ]);
  if(!env.strategy){
    const { scenario } = await inquirer.prompt([
      {
        type: "input",
        name: "scenario",
        message: `Scenario:  ${env.scenario ? `(${env.scenario})` : ""}`
      }
    ]);
    const { service } = await inquirer.prompt([
      {
        type: "input",
        name: "service",
        message: `Service:  ${env.service ? `(${env.service})` : ""}`
      }
    ]);
    updateConfig({
      scenario: scenario || env.scenario,
      service: service || env.service,
    })
  }else{
    const { strategy } = await inquirer.prompt([
      {
        type: "input",
        name: "strategy",
        message: `Strategy:  ${env.strategy ? `(${env.strategy})` : ""}`
      }
    ]);
    const { scenariosConfig } = await inquirer.prompt([
      {
        type: "input",
        name: "scenariosConfig",
        message: `ScenariosConfig:  ${env.scenariosConfig ? `(${env.scenariosConfig})` : ""}`
      }
    ]);
    updateConfig({
      strategy: strategy || env.strategy,
      scenariosConfig: scenariosConfig || env.scenariosConfig,
    })
  }


  execute("basename `yarn generate:legals`", async (legals)=>{
    console.log("GENERATING LEGALS")
  });



}

preBuild();