import fs from "fs";
import path from "path";
import axios from "axios";
import S3 from "aws-sdk/clients/s3"
import slugify from "slugify";
import mime from "mime-types";
import * as envfile from "envfile";
import moment from "moment";
import { exec } from "child_process";

const execute = (command, callback)=>{
  exec(command, (error, stdout, stderr) => callback(stdout));
};
const env = envfile.parseFileSync(path.resolve("", ".env"));
const BUCKET_NAME = "mobirun";

const client = new S3({
  accessKeyId: process.env.osui_aws_access_key_id,
  secretAccessKey: process.env.osui_secret_access_key
});

function publishPage(
  s3UploadDir: string,
  s3StageFile: string,
  s3ProdFile: string
) {
  const s3FullPathStage = s3UploadDir + s3StageFile;
  const s3FullPathProd = s3UploadDir + s3ProdFile;
  console.log(`Publishing ${s3StageFile} -> ${s3ProdFile}`);
  console.log(`${BUCKET_NAME}/${s3FullPathStage}`);

  client
    .copyObject({
      Bucket: BUCKET_NAME,
      CopySource: `${BUCKET_NAME}/${s3FullPathStage}`,
      Key: s3FullPathProd,
      ACL: 'public-read'
    })
    .promise()
    .then(res => {
      console.log(res);
    })
    .catch(err => {
      console.error(err);
    });
}

function cacheAndPublishPage(
  s3UploadDir: string,
  s3StageFile: string,
  s3ProdFile: string
) {
  const now = moment()
    .format("YYYYMMDDHHMMSS")
    .toString();
  const cachePageName = `${s3ProdFile}.${now}`;
  const s3FullPathProd = s3UploadDir + s3ProdFile;

  console.log(`Caching current ${s3ProdFile} -> ${cachePageName}`);
  client
    .copyObject({
      Bucket: BUCKET_NAME,
      CopySource: `${BUCKET_NAME}/${s3FullPathProd}`,
      Key: s3UploadDir + cachePageName,
      ACL: 'public-read'
    })
    .promise()
    .then(() =>
      client
        .deleteObject({
          Bucket: BUCKET_NAME,
          Key: s3FullPathProd
        })
        .promise()
        .then(() => publishPage(s3UploadDir, s3StageFile, s3ProdFile))
    )
    .catch(err => {
      console.error(err);
    });
}

async function main() {
  const { page, country, scenario, strategy, scenariosConfig } = env;
  const slugifiedString = scenario ? slugify(scenario, { lower: true }) : slugify(`${strategy}_${scenariosConfig}`, { lower: true });
  const s3UploadDir = `os-ui/static/${page}/html/`;
  const s3ProdFile = `${country}-${slugifiedString}-production.html`;
  const s3StageFile = `${country}-${slugifiedString}-staging.html`;

  const params = {
    Bucket: BUCKET_NAME,
    Key: s3UploadDir + s3ProdFile
  };

  // Check whether file exist or not
  // and perform appropriate action
  client.headObject(params, (err, data) => {
    if (err && err.statusCode === 404) {
      publishPage(s3UploadDir, s3StageFile, s3ProdFile);
    } else {
      cacheAndPublishPage(s3UploadDir, s3StageFile, s3ProdFile);
    }
  });
}

const apiUrl = "http://127.0.0.1:3030";
//const apiUrl = "https://c1.ouisys.com";
main()
  .then(()=>{
      //save the page upload to the database
      const { country, page, scenario, strategy, scenariosConfig } = env;
      const slugifiedString = scenario ? slugify(scenario, { lower: true }) : slugify(`${strategy}_${scenariosConfig}`, { lower: true });
      const s3UploadDir = `os-ui/static/${page}/html/`;
      const s3ProdFile = `${country}-${slugifiedString}-production.html`;
      const s3Page = `https://${s3UploadDir}${s3ProdFile}`
      
      const releasePage = (data)=>{
        axios({
          url:`${apiUrl}/api/v1/release-page`,
          method: 'post',
          data: {
            html_url:s3Page,
            page_upload_id:data.id,
            username:data.git_username
          }
        }).then(function(response) {
          console.log("release page response", response)
          console.log("Page released", `http://c1.ouisys.com/preview/?country=${country}&page=${page}&scenario=${slugifiedString}`)
          return response.data;
        })
        .catch((err)=>{
          console.error(err);
        })
      };

      axios({
        url:`${apiUrl}/api/v1/get-page`,
        method: 'get',
        params: {
          page,
          country,
          scenario:slugifiedString,
          strategy,
          scenarios_config:scenariosConfig
        }
      }).then(function(response) {
        return response.data;
      })
      .then((data)=>{
        releasePage(data);
      })
      .catch((err)=>{
        console.error(err);
      })
      
  })
  .catch(console.error);
