import fs from "fs";
import fsExt from "fs-extra"
import path from "path";
import slugify from "slugify"
const envfile = require("envfile");
const env = envfile.parseFileSync(path.resolve("", ".env"));
const { renderToString } = require("react-dom/server");
const React = require("react");
require("dotenv").config();

const { country, page, scenario, strategy, scenariosConfig } = env;
const NOREACT = "true" == process.env.noReact || "true" == process.env.NOREACT

let slugifiedString = scenario? slugify(scenario, { lower: true }) :  slugify(`${strategy}_${scenariosConfig}`, { lower: true });
let stageIndexFileName = `${country}-${slugifiedString}-staging.html`;
const entryHtmlFile = stageIndexFileName;

const inputFileName = path.join(path.resolve("", `./dist/static/${page}/html`, `index.html`));
const outputFileName = path.join(path.resolve("", `./dist/static/${page}/html`, stageIndexFileName));

console.log(stageIndexFileName)
console.log(outputFileName)

async function copyAssets() {
  await fsExt.existsSync(path.resolve("", `./assets`))
    ? fsExt.copy(path.resolve("", `./assets`), path.resolve("", `./dist/static/${page}/`))
    : Promise.resolve();
}

async function mainWithoutReact() {
  await copyAssets();
  await fs.promises.rename(inputFileName, outputFileName)
}

async function mainWithReact() {
  await copyAssets()
  const ReactApp = require(path.resolve("", `./dist/static/${page}/ssr/main`)).default

  const reactElement = React.createElement(ReactApp);

  console.log("DIR", __dirname)

  const file = fs.readFileSync(inputFileName, "utf8");
  const [head, tail] = file.split("{react-app}");

  var ssr = '';
  try {
    ssr = renderToString(reactElement)
  } catch (ex) {
    console.warn(ex)
  }

  fs.writeFileSync(outputFileName, head + ssr + tail, 'utf8')
}


if(NOREACT) {
  mainWithoutReact()
  .then(() => console.log('✅ NOREACT build is done!'))
  .catch(err => console.error(err))
} else {
  mainWithReact()
    .then(() => console.log('✅React build is done!'))
    .catch(err => console.error(err))
}