import fs from 'fs'
import axios from "axios";
import path from 'path'
import glob from "glob"
import S3 from "aws-sdk/clients/s3"
import slugify from "slugify";
import asyncPool from "tiny-async-pool";
import mime from 'mime-types'
import * as envfile from "envfile";
import { exec } from "child_process";

const execute = (command, callback)=>{
  exec(command, (error, stdout, stderr) => callback(stdout));
};

const env = envfile.parseFileSync(path.resolve("", '.env'));

const toPromise = (f) => (...args) => new Promise((resolve, reject) => {
  f(...args, (err, res) => !!err ? reject(err) : resolve(res))
})

//console.log("ENV", env);

const client = new S3({
  accessKeyId: process.env.osui_aws_access_key_id,
  secretAccessKey: process.env.osui_secret_access_key,
});

const { country, page, scenario, strategy, scenariosConfig } = env;
const slugifiedString = scenario ? slugify(scenario, { lower: true }) : slugify(`${strategy}_${scenariosConfig}`, { lower: true });


const upload = ({key, file}) => {
  console.log("KEY", key)
  console.log("FILE", file)
  const cacheControl = /\.html$/.test(file) ? 'max-age=1' : 'max-age=604800'
  console.log('🔼', key, cacheControl)
  return client.upload({
    Bucket: 'mobirun',
    Key: `os-ui${key}`,
    Body: fs.createReadStream(file),
    ACL: 'public-read', 
    ContentType: mime.contentType(path.extname(file)) as string,
    CacheControl: cacheControl
  }).promise()
}


async function main () {
  const root = path.resolve('dist')
  
  const files = <string[]> await toPromise(glob)(
    root + "/static/**/*.*"
    , { nodir: true} 
  );

  const htmlFileName = `${country}-${slugifiedString}-staging.html`;
  const s3HtmlUrl = `https://s3.eu-central-1.amazonaws.com/mobirun/os-ui/static/${page}/html/${htmlFileName}`;
  const uploadResult = await (<any>asyncPool)(4, files.map(f => ({key: f.replace(root, ''), file: f})), upload);
  console.log(`✅  # ${uploadResult.length} Uploaded!`)
  //console.log(`🔗 # http://staging.ouisys.com/preview/?country=${country}&page=${page}&scenario=${slugifiedScenario}`);
  console.log(s3HtmlUrl);

  //const apiUrl = "http://127.0.0.1:3030";
  const apiUrl = "https://c1.ouisys.com";
  //save the page upload to the database
  const savePageUploadRecord = ()=>{
    execute("git config --global user.name", (name)=>{
        execute("git config --global user.email", (email)=>{
          execute("git config --get remote.origin.url", async (git_origin)=>{
            const data = { 
              page,
              country,
              scenario: scenario ? slugifiedString : null,
              strategy: strategy || null,
              scenarios_config:scenariosConfig || null,
              html_url:s3HtmlUrl,
              git_username: name.replace("\n", ""),
              //email: email.replace("\n", ""),
              git_origin: git_origin.replace("\n", "") 
            };
            try{
              const res = await axios({
                url:`${apiUrl}/api/v1/upload-page`,
                method: 'post',
                data: {
                  env_dump:JSON.stringify(env),
                  ...data
                }
              });

              if(res){
                const campaign = await axios({
                  //url:'https://c1.ouisys.com/api/v1/create-campaign',
                  url:`${apiUrl}/api/v1/create-campaign`,
                  method: 'post',
                  data: {
                    affid:"SAM",
                    ...data
                  }
                });
              


              console.log(`🔗 # http://staging.ouisys.com/${campaign.data.xcid}`);

              console.log("upload record saved! 😁")
            }
              return res;

            }catch(err){
              //need tofix 
              if(err.response.status !== 500){
                
                throw console.log(err);
              }
            }
          });
        });
    });
  };

  savePageUploadRecord();     
  //create a campaign
  const createCampaign = ()=>{
    execute("git config --global user.name", (name)=>{
        execute("git config --global user.email", (email)=>{
          execute("git config --get remote.origin.url", async (git_origin)=>{
            const data = { 
              page,
              country,
              scenario: scenario ? slugifiedString : null,
              strategy: strategy || null,
              scenarios_config:scenariosConfig || null,
              affid:"SAM",
              comments:`Build upload by name`
            };
            try{
              const res = await axios({
                url:`${apiUrl}/api/v1/create-campaign`,
                method: 'post',
                data: data
              });
              console.log("campaign created under affid - SAM!");
              console.log(`STAGING URL 🔗 # http://staging.ouisys.com/${res.data.xcid}`);
              return res;

            }catch(err){
              console.error(err)
              //need tofix 
              if(err.response.status !== 500){
                console.log("Failed to save upload record", err)
              }
            }
          });
        });
    });
  };

  createCampaign();
}


main().then(()=>console.log("")).catch(console.error)
