const path = require('path')
const webpack = require('webpack')
const Dotenv = require('dotenv-webpack');
const envfile = require("envfile");
const env = envfile.parseFileSync(path.resolve("", ".env"));
const FaviconsWebpackPlugin = require('favicons-webpack-plugin');

require('dotenv').config()

const page = env.page
const NOREACT = "true" == process.env.noReact || "true" == process.env.NOREACT
//const NOEXTERNAL = "true" == process.env.NOEXTERNAL

const urlModule = {
  test: /\.(pdf|jpg|jpeg|png|gif|svg|ico|woff|woff2|ttf|eot)$/,
  use: [
    {
      loader: 'url-loader',
      options: {
        limit: 12000,
        name: `static/${page}/files/[hash].[ext]`,
      }
    },
  ]
}

const optimizeImgs = {
  test: /\.(jpg|png|gif|svg)$/,
  loader: 'image-webpack-loader',
  // Specify enforce: 'pre' to apply the loader
  // before url-loader/svg-url-loader
  // and not duplicate it in rules with them
  enforce: 'pre'
}

const puresModule = {
  test: /\.purs$/,
  exclude: /node_modules/,
  loader: 'purs-loader',
  options: {
    src: [
      'bower_components/purescript-*/src/**/*.purs',
      'src/**/*.purs'
    ],
    pscIde: true
  }
}

const tsModule = {
  test: /\.(js|jsx|ts|tsx)$/,
  include: [path.resolve('src')],
  exclude: /node_modules/,
  // loader: 'ts-loader',
  loader: 'babel-loader'
}

const mkTypeScriptCSSLoader = camelCase => ({
  loader: 'typings-for-css-modules-loader',
  options: {
    modules: true,
    namedExport: true,
    sourceMap: true,
    localIdentName: '[path][name]__[local]--[hash:base64:5]',
    camelCase,
    importLoaders: 3,
  }
})

const postCSSLoader = {
  loader: 'postcss-loader',
  options: {
    config: {
      path: __dirname
    },
    sourceMap: true,
  },
}

const lessLoader = {
  loader: 'less-loader',
  options: {
    sourceMap: true,
  },
}

const stylusLoader = {
  loader: 'stylus-loader',
  options: {
    sourceMap: true,
  },
}

const sassLoader = {
  loader: 'sass-loader',
  options: {
    sourceMap: true,
  },
}


// const externals = (NOREACT || NOEXTERNAL) ? {} : {
//   react: 'React',
//   'react-dom': 'ReactDOM',
// }

const externals = {} 

const resolve = {
  alias: {
    moment: 'moment/moment.js',
  },
  modules: [
    'node_modules',
    'bower_components'
  ],
  symlinks: false,

  extensions: ['.tsx', '.ts', '.js', '.purs']
}

const definePlugin = new webpack.DefinePlugin({
  'process.env': {
    'NODE_ENV': JSON.stringify(process.env.NODE_ENV),
    'country': JSON.stringify(process.env.country || ''),
    'scenario': JSON.stringify(process.env.scenario || ''),
    'service': JSON.stringify(process.env.service || ''),
    'strategy': JSON.stringify(process.env.strategy || ''),
    'scenariosConfig': JSON.stringify(process.env.scenariosConfig || ''),
    'page': JSON.stringify(process.env.page || ''),
    'title': JSON.stringify(process.env.title || ''),
    'client': JSON.stringify(process.env.client || ''),
    'pacman_server': JSON.stringify(process.env.pacman_server || "https://de-pacman.sam-media.com/api/v2/mstore"),
    // 'api_root': JSON.stringify(process.env.api_root || ''),
    // 'finance_email': JSON.stringify(process.env.finance_email || '')
  }
})

const modules = {
  'purs': puresModule,
  'ts': tsModule,
  'url': urlModule,
  optimizeImgs
}

const loaders = {
  'typings-for-css': mkTypeScriptCSSLoader(false),
  'typings-for-css-camelCase': mkTypeScriptCSSLoader(true),
  'postcss': postCSSLoader,
  'less': lessLoader,
  'stylus': stylusLoader,
  'sass': sassLoader
}

const plugins = {
  define: definePlugin,
  dotEnv: new Dotenv(),
  favicon: (process.env.NODE_ENV === "production") ? new FaviconsWebpackPlugin({
    logo:"./src/assets/imgs/favicon.png",
    prefix:`static/${page}/files/`,
    icons: {
      android: false,
      appleIcon: false,
      appleStartup: false,
      coast: false,
      favicons: true,
      firefox: false,
      opengraph: false,
      twitter: false,
      yandex: false,
      windows: false
    }
  }) : new FaviconsWebpackPlugin("./assets/imgs/favicon.png")
}

module.exports = {
  modules,
  plugins,
  loaders,
  resolve,
  externals
}